using System;
using System.Collections.Generic;
using System.Text;

namespace Vevo.Base.Domain
{
    [Serializable]
    public class SearchFilter
    {
        public enum SearchFilterType
        {
            None,
            Text,
            Boolean,
            ValueRange,
            Date
        }

        #region Factory Class

        public class Factory
        {
            private SearchFilterType _filterType = SearchFilterType.None;
            private string _fieldName = String.Empty;
            private string _value1 = String.Empty;
            private string _value2 = String.Empty;

            public Factory WithFilterType( SearchFilterType value ) { _filterType = value; return this; }
            public Factory WithFieldName( string value ) { _fieldName = value; return this; }
            public Factory WithValue1( string value ) { _value1 = value; return this; }
            public Factory WithValue2( string value ) { _value2 = value; return this; }
            public SearchFilter Create()
            {
                return new SearchFilter( _filterType, _fieldName, _value1, _value2 );
            }
        }

        public static Factory GetFactory() { return new Factory(); }

        #endregion

        # region Private
        private SearchFilterType _filterType;
        private string _fieldName;
        private string _value1;
        private string _value2;
        #endregion

        #region Constructors

        public SearchFilter()
        {
            Clear();
        }

        public SearchFilter(
            SearchFilterType filterType,
            string fieldName,
            string value1,
            string value2
            )
        {
            _filterType = filterType;
            _fieldName = fieldName;
            _value1 = value1;
            _value2 = value2;
        }

        #endregion

        #region Public Properties
        public SearchFilterType FilterType { get { return _filterType; } }
        public string FieldName { get { return _fieldName; } }
        public string Value1 { get { return _value1; } }
        public string Value2 { get { return _value2; } }
        #endregion

        #region Public Methods

        public void Clear()
        {
            _filterType = SearchFilterType.None;
            _fieldName = String.Empty;
            _value1 = String.Empty;
            _value2 = String.Empty;
        }

        // Need to create unique string for DataCache class
        public override string ToString()
        {
            string result = _filterType.ToString();
            if (!String.IsNullOrEmpty( _fieldName ))
                result += ", Field Name: " + _fieldName;
            if (!String.IsNullOrEmpty( _value1 ))
                result += ", Value1: " + _value1;
            if (!String.IsNullOrEmpty( _value2 ))
                result += ", Value2: " + _value2;

            return result;
        }
        #endregion
    }
}
