using System;
using System.Collections.Generic;
using System.Text;
using Vevo.Domain.DataInterfaces;
using Vevo.Base.Data.Caching;
using Vevo.Domain.Payments;
using Vevo.Domain;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class PayPalIpnRepository : IPayPalIpnRepository
    {
        #region Private

        private const string RepositoryName = "PayPalIpnRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetIpnIDByReferenceIDSignature = "GetIpnIDByReferenceID";
        private const string TableName = "PayPalIPN";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( TableName );
        private PayPalIpnRepositoryNoCache _repository = new PayPalIpnRepositoryNoCache();

        #endregion

        #region Constructors

        public PayPalIpnRepository()
        {
        }

        public PayPalIpnRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public PayPalIpn Save( PayPalIpn payPalIpn )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            PayPalIpn newPayPalIpn = _repository.Save( payPalIpn );
            _dataCache.ClearRepositoryCache();
            return newPayPalIpn;
        }

        public PayPalIpn GetOne( string ipnID )
        {
            PayPalIpn result = (PayPalIpn) _dataCache.GetValue( GetOneSignature, _cacheDependency, ipnID );

            if (result == null)
            {
                result = _repository.GetOne( ipnID );
                _dataCache.SetValue( GetOneSignature, result, ipnID );
            }

            return result;
        }

        public string GetIpnIDByReferenceID( string referenceID )
        {
            object result = _dataCache.GetValue( GetIpnIDByReferenceIDSignature, _cacheDependency, referenceID );

            if (result == null)
            {
                result = _repository.GetIpnIDByReferenceID( referenceID );
            }

            return result.ToString();
        }

        public void Delete( string ipnID )
        {
            _repository.Delete( ipnID );
            _dataCache.ClearRepositoryCache();
        }

        #endregion

    }
}
