﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Vevo.Domain.Products;

namespace Vevo.Domain.Base
    {[Serializable]
    public class MetaInformationCollection<T> : LocaleCollection<T> where T : ProductMetaInformation, new() 
    {
        #region Private

        private void LoadMissingLocaleFromRepository()
        {
            Product product = (Product) _localeLoader;
            IList<ILocale> list = product.GetMetaInformations();
            foreach (T item in list)
            {
                if (SearchList( item.CultureID, item.StoreID ) == null)
                    this.InnerList.Add( item );
            }

            _loaded = true;
        }

        private T SearchList( string cultureID,string storeID )
        {
            foreach (T item in this.InnerList)
            {
                if ((item.CultureID == cultureID) && (item.StoreID == storeID))
                    return item;
            }

            return default( T );
        }

        #endregion

        #region Public Constructors

        public MetaInformationCollection( Product localeLoader ): base( localeLoader)
        {
            _localeLoader = localeLoader;
        }
        #endregion

        #region Public Methods

        public new void Load()
        {
            if (!_loaded)
                LoadMissingLocaleFromRepository();
        }

        public new object Clone()
        {
            MetaInformationCollection<T> newObject = new MetaInformationCollection<T>( (Product)_localeLoader );
            newObject._loaded = _loaded;

            foreach (T item in this.InnerList)
                newObject.InnerList.Add( (T) item.Clone() );

            return newObject;
        }

        public new void Add( T item )
        {
            if (!_loaded)
                LoadMissingLocaleFromRepository();

            if (SearchList( item.CultureID,item.StoreID ) == null)
                this.InnerList.Add( item );
        }

        #endregion 

        #region Public Properties

        public new T this[string cultureID]
        {
            get
            {
                T item = SearchList( cultureID );
                if (item == null)
                {
                    if (!_loaded)
                        LoadMissingLocaleFromRepository();

                    item = GetNoLoad( cultureID );
                }

                return item;
            }
        }

        #endregion

        #region IEnumerable<T> Members

        public new IEnumerator<T> GetEnumerator()
        {
            Load();
            return InnerList.GetEnumerator();
        }

        #endregion

    }
}
