﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Vevo.Domain.Products;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.Configurations
{
    public static class SeoVariable
    {
        private const string ProductVariable = "[ProductSKU], [ProductName], [ProductShortDescription], [SiteName]";
        private const string CategoryVariable = "[CategoryName], [CatgoryParentList], [CategoryDescription], [SiteName]";
        private const string DepartmentVariable = "[DepartmentName], [DepartmentParentList], [DepartmentDescription], [SiteName]";

        public static string[] GetProductVariable()
        {
            return ProductVariable.Trim().Split( ',' );
        }

        public static string[] GetCategoryVariable()
        {
            return CategoryVariable.Trim().Split( ',' );
        }

        public static string[] GetDepartmentVariable()
        {
            return DepartmentVariable.Trim().Split( ',' );
        }

        public static string ReplaceProductVariable( Product product, Culture culture, string content )
        {
            content = content.Replace( "[SiteName]",
                DataAccessContext.Configurations.GetValue( culture.CultureID, "SiteName" ) );
            content = content.Replace( "[ProductSKU]", product.Sku );
            content = content.Replace( "[ProductName]", product.Name );
            content = content.Replace( "[ProductShortDescription]", product.ShortDescription );

            return content;
        }

        public static string ReplaceCategoryVariable( Category category, Culture culture, string content )
        {
            content = content.Replace( "[SiteName]",
                DataAccessContext.Configurations.GetValue( culture.CultureID, "SiteName" ) );
            content = content.Replace( "[CategoryName]", category.Name );
            content = content.Replace( "[CategoryDescription]", category.Description );
            content = content.Replace( "[CatgoryParentList]", category.CreateParentCategoryPathList() );

            return content;
        }

        public static string ReplaceDepartmentVariable( Department department, Culture culture, string content )
        {
            content = content.Replace( "[SiteName]",
                DataAccessContext.Configurations.GetValue( culture.CultureID, "SiteName" ) );
            content = content.Replace( "[DepartmentName]", department.Name );
            content = content.Replace( "[DepartmentDescription]", department.Description );
            content = content.Replace( "[DepartmentParentList]", department.CreateParentDepartmentPathList() );

            return content;
        }
    }

}
