using System;
using System.Data;
using System.Collections.Generic;
using System.Configuration;
using Vevo.Domain;
using Vevo.Domain.Configurations;
using Vevo.Domain.Orders;
using Vevo.Domain.Users;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.Discounts
{
    /// <summary>
    /// Summary description for DiscountCalculator
    /// </summary>
    public class DiscountCalculator
    {
        #region Private

        private QuantityDiscountCalculator _quantityDiscountCalculator;


        private IList<decimal> ListAdd( IList<decimal> list1, IList<decimal> list2 )
        {
            IList<decimal> newList = new List<decimal>();

            for (int i = 0; i < list1.Count; i++)
            {
                newList.Add( list1[i] + list2[i] );
            }

            return newList;
        }

        #endregion


        #region Constructors

        public DiscountCalculator()
        {
            _quantityDiscountCalculator = new QuantityDiscountCalculator();
        }

        #endregion


        #region Public Methods
        public decimal GetDiscount(
            CartItemGroup cartItemGroup,
            Coupon coupon,
            Customer customer,
            //decimal shippingCost,
            //decimal handlingFee,
            out IList<decimal> discountLines )
        {
            //discountLines = null;
            //WholesaleStatus wholesaleStatus = customer.GetWholesaleStatus( _configurations );

            //return Currency.RoundAmount(
            //    _quantityDiscountCalculator.GetDiscount( cartItemGroup, wholesaleStatus, out discountLines )
            //    + coupon.GetDiscount( cartItemGroup, customer, discountLines, out discountLines ) );

            WholesaleStatus wholesaleStatus = customer.GetWholesaleStatus( DataAccessContext.Configurations );

            IList<decimal> quantityDiscountLines, couponDiscountLines;

            decimal discount = Currency.RoundAmount(
                _quantityDiscountCalculator.GetDiscount( cartItemGroup, wholesaleStatus, out quantityDiscountLines )
                + coupon.GetDiscount( cartItemGroup, customer, out couponDiscountLines ) );

            discountLines = ListAdd( quantityDiscountLines, couponDiscountLines );

            return discount;
        }

        public decimal GetDiscountNoCouponExpiration(
            CartItemGroup cartItemGroup,
            Coupon coupon,
            Customer customer,
            //decimal shippingCost,
            //decimal handlingFee,
            out IList<decimal> discountLines )
        {
            WholesaleStatus wholesaleStatus = customer.GetWholesaleStatus( DataAccessContext.Configurations );

            IList<decimal> quantityDiscountLines, couponDiscountLines;

            decimal discount = Currency.RoundAmount(
                _quantityDiscountCalculator.GetDiscount( cartItemGroup, wholesaleStatus, out quantityDiscountLines )
                + coupon.GetDiscountNoExpiration( cartItemGroup, customer, out couponDiscountLines ) );

            discountLines = ListAdd( quantityDiscountLines, couponDiscountLines );

            return discount;
        }

        #endregion

    }
}
