﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Vevo.Domain.Products;
using Vevo.Shared.Utilities;
using Vevo.Shared.Domain.SystemServices;
using System.Collections;
using System.Data;
using Vevo.Shared.DataAccess;
using System.IO;
using LumenWorks.Framework.IO.Csv;
using Vevo.Domain;
using Vevo.Domain.Tax;
using Vevo.Domain.Stores;
using Vevo.Domain.Base;
using Vevo.Domain.Exceptions;
using Vevo.Domain.ImportExport;
using System.Drawing;
using Vevo.Domain.Products.BackOffice;
using Vevo;

namespace Vevo.Domain.ImportExport
{
    public class LargeImporter
    {
        #region Private
        private Dictionary<string, string> _skuDatabase = new Dictionary<string, string>();
        private const int _categoryIDIndex = 0;
        private const int _departmentIDIndex = 1;
        private const int _nameIndex = 2;
        private const int _shortDescriptionIndex = 3;
        private const int _longDescriptionIndex = 4;
        private const int _skuIndex = 5;
        private const int _manufacturerIndex = 6;
        private const int _modelIndex = 7;
        private const int _manufacturerPartNumber = 8;
        private const int _upc = 9;
        private const int _useInventory = 10;
        private const int _stockIndex = 11;
        private const int _weightIndex = 12;
        private const int _priceIndex = 13;
        private const int _retailPriceIndex = 14;
        private const int _wholesalePrice1Index = 15;
        private const int _wholesalePrice2Index = 16;
        private const int _wholesalePrice3Index = 17;
        private const int _keywordsIndex = 18;
        private const int _other1Index = 19;
        private const int _other2Index = 20;
        private const int _other3Index = 21;
        private const int _other4Index = 22;
        private const int _other5Index = 23;
        private const int _isEnabledIndex = 24;
        private const int _taxClassID = 25;

        private const int _isTodaySpecialIndex = 26;
        private const int _isDownloadableIndex = 27;
        private const int _downloadPathIndex = 28;
        private const int _relatedProducts = 29;


        private const int _largeImage1 = 30;
        private const int _isZoom1 = 31;
        private const int _isEnlarge1 = 32;
        private const int _largeImage2 = 33;
        private const int _isZoom2 = 34;
        private const int _isEnlarge2 = 35;
        private const int _largeImage3 = 36;
        private const int _isZoom3 = 37;
        private const int _isEnlarge3 = 38;
        private const int _largeImage4 = 39;
        private const int _isZoom4 = 40;
        private const int _isEnlarge4 = 41;
        private const int _largeImage5 = 42;
        private const int _isZoom5 = 43;
        private const int _isEnlarge5 = 44;
        private const int _largeImage6 = 45;
        private const int _isZoom6 = 46;
        private const int _isEnlarge6 = 47;
        private const int _largeImage7 = 48;
        private const int _isZoom7 = 49;
        private const int _isEnlarge7 = 50;
        private const int _largeImage8 = 51;
        private const int _isZoom8 = 52;
        private const int _isEnlarge8 = 53;
        private const int _largeImage9 = 54;
        private const int _isZoom9 = 55;
        private const int _isEnlarge9 = 56;
        private const int _largeImage10 = 57;
        private const int _isZoom10 = 58;
        private const int _isEnlarge10 = 59;
        private const int _isGiftCertificate = 60;
        private const int _isFreeShippingCost = 61;
        private const int _isFixedPrice = 62;

        private const int _giftAmount = 63;
        private const int _expiredType = 64;
        private const int _fixExpiredDate = 65;
        private const int _numberOfDay = 66;
        private const int _isElectronic = 67;

        private const int _useOptionIndex = 68;
        private const int _optionGroup1IDIndex = 69;
        private const int _useStock1Index = 70;
        private const int _optionGroup2IDIndex = 71;
        private const int _useStock2Index = 72;
        private const int _optionGroup3IDIndex = 73;
        private const int _useStock3Index = 74;
        private const int _optionGroup4IDIndex = 75;
        private const int _useStock4Index = 76;
        private const int _optionGroup5IDIndex = 77;
        private const int _useStock5Index = 78;

        private const int _optionItem1IDIndex = 79;
        private const int _optionItem2IDIndex = 80;
        private const int _optionItem3IDIndex = 81;
        private const int _optionItem4IDIndex = 82;
        private const int _optionItem5IDIndex = 83;

        private const int _isRecurring = 84;
        private const int _recurringInterval = 85;
        private const int _recurringIntervalUnit = 86;
        private const int _recurringNumberOfCycles = 87;
        private const int _recurringNumberOfTrialCycles = 88;
        private const int _recurringTrialAmount = 89;

        private const int _metaKeyword = 90;
        private const int _metaDescription = 91;

        private const int _isAffiliate = 92;

        private DateTime _timeStart;
        private DateTime _timeEnd;

        private Culture _culture;
        private IFileManager _fileManager;

        private ProductImportStatus _status;

        private ArrayList _importedSkuList = new ArrayList();
        private Dictionary<string, string> _updateRelatedIDList = new Dictionary<string, string>();

        private IList<Store> _storeList;
        private ArrayList _categoryIDList;
        private ArrayList _taxClassIDList;
        private DataTable _allCategoriesListTable;
        private ArrayList _categoryVisibleIDList = new ArrayList();
        private ArrayList _departmentIDList;

        private string _firstTaxClassID;
        private string _selectedStoreID;

        private bool IsCategoryExist( string categoryIDs )
        {
            string[] categoryIDsplit = ConvertToCategoryIDs( categoryIDs );

            foreach (string categoryID in categoryIDsplit)
            {
                if (!_categoryIDList.Contains( categoryID ))
                    return false;
            }
            return true;
        }

        private bool IsDepartmentExist( string departmentIDs )
        {
            string[] departmentIDsplit = ConvertToCategoryIDs( departmentIDs );

            if (departmentIDsplit.Count() > 0)
            {
                foreach (string id in departmentIDsplit)
                {
                    if (!_departmentIDList.Contains( id ))
                        return false;
                }
            }
            return true;
        }
        private bool IsTaxClassExist( string taxClassID )
        {
            if (String.IsNullOrEmpty( taxClassID ))
                return true;

            string[] ignoreList = { "", "yes", "no", "true", "false", "0" };
            foreach (string item in ignoreList)
            {
                if (taxClassID.ToLower().Trim() == item)
                    return true;
            }

            if (_taxClassIDList.Contains( taxClassID ))
                return true;

            return false;
        }

        private string GetRelatedProductIDBySku( string sku )
        {
            string id = GetProductIDBySku( sku, false );

            if (!String.IsNullOrEmpty( sku ))
            {
                return id;
            }
            else
            {
                return DataAccess.ExecuteScalar(
                "SELECT ProductID " + "FROM Product " + "WHERE SKU = @sku; ",
                DataAccess.CreateParameterString( sku ) );
            }
        }

        private string ConvertToTaxClassID( string value )
        {
            if (!string.IsNullOrEmpty( value ))
            {
                switch (value.ToLower())
                {
                    case "true":
                    case "yes":
                        return _firstTaxClassID;

                    case "no":
                    case "false":
                        return TaxClass.NonTaxClassID;

                    default:
                        return value;
                }
            }
            else
                return TaxClass.NonTaxClassID;
        }

        private string[] ConvertToCategoryIDs( string source )
        {
            string[] strSplit = source.Split( new char[] { '|' }, StringSplitOptions.RemoveEmptyEntries );

            for (int i = 0; i < strSplit.Length; i++)
            {
                strSplit[i] = strSplit[i].Trim();
            }

            return strSplit;
        }

        private bool ConvertToBool( string value )
        {
            int intValue;
            if (Int32.TryParse( value, out intValue ))
                return (intValue != 0);

            switch (value.ToLower())
            {
                case "true":
                case "yes":
                    return true;

                default:
                    return false;
            }
        }

        private Product SetProductMetaInformation(
            Product product, string storeID, string metaKeyword, string metaDescription )
        {
            //IList<Store> stores = DataAccessContext.StoreRepository.GetAll( "StoreID" );
            // if (product.GetMetaInformation("0") == null)
            if (product.ProductID == "0")
            {
                // Purge
                //product.SetMetaKeyword("0", metaKeyword);
                //product.SetMetaDescription("0", metaDescription);
                product.SetMetaInfo( "0", metaKeyword, metaDescription, true );

                if (storeID != "0")
                {
                    product.SetMetaInfo( storeID, metaKeyword, metaDescription, true );
                }
                //foreach (Store store in _storeList)
                //{
                //    //product.SetMetaKeyword(store.StoreID, metaKeyword);
                //    //product.SetMetaDescription(store.StoreID, metaDescription);

                //    product.SetMetaInfo(store.StoreID, metaKeyword, metaDescription, true);
                //}
            }
            else
            {
                // Overwrite
                if (storeID == "0")
                {
                    //product.SetMetaKeyword("0", metaKeyword);
                    //product.SetMetaDescription("0", metaDescription);

                    product.SetMetaInfo( "0", metaKeyword, metaDescription );


                    foreach (Store store in _storeList)
                    {
                        ProductMetaInformation meta = product.GetMetaInformation( store.StoreID );
                        if (meta != null)
                        {
                            if (meta.UseDefaultMetaKeyword == true)
                            {
                                product.SetMetaKeyword( store.StoreID, metaKeyword );
                            }
                            if (meta.UseDefaultMetaDescription == true)
                            {
                                product.SetMetaDescription( store.StoreID, metaDescription );
                            }

                        }

                        //if (meta == null)
                        //{
                        //    //product.SetMetaKeyword(store.StoreID, metaKeyword);
                        //    //product.SetMetaDescription(store.StoreID, metaDescription);

                        //    product.SetMetaInfo(store.StoreID, metaKeyword, metaDescription, true);
                        //}
                        //else
                        //{
                        //if (meta.UseDefaultMetaKeyword == true)
                        //{
                        //    product.SetMetaKeyword(store.StoreID, metaKeyword);
                        //}
                        //if (meta.UseDefaultMetaDescription == true)
                        //{
                        //    product.SetMetaDescription(store.StoreID, metaDescription);
                        //}
                        // }
                    }
                }
                else
                {
                    //product.SetUseDefaultValueMetaKeyword(storeID, false);
                    //product.SetUseDefaultValueMetaDescription(storeID, false);
                    //product.SetMetaKeyword(storeID, metaKeyword);
                    //product.SetMetaDescription(storeID, metaDescription);

                    product.SetMetaInfo( storeID, metaKeyword, metaDescription, false, false );
                }
            }

            return product;
        }


        //private Product SetProductMetaInformation(
        //    Product product, string storeID, string metaKeyword, string metaDescription)
        //{

        //    //metaInfoDict Key-> storeID 
        //    Dictionary<string, ProductMetaInformation> metaInfoDict = new Dictionary<string, ProductMetaInformation>();
        //    foreach (ProductMetaInformation meta in product.MetaInformations)
        //    {
        //        metaInfoDict.Add(meta.StoreID, meta);
        //    }

        //    if (!metaInfoDict.ContainsKey("0"))
        //    {
        //        // Purge
        //        ProductMetaInformation newMeta = new ProductMetaInformation();
        //        newMeta.StoreID = "0";
        //        newMeta.MetaKeyword = metaKeyword;
        //        newMeta.MetaDescription = metaDescription;

        //        metaInfoDict.Add("0", newMeta);

        //        foreach (Store store in _storeList)
        //        {
        //            newMeta = new ProductMetaInformation();
        //            newMeta.StoreID = store.StoreID;
        //            newMeta.MetaKeyword = metaKeyword;
        //            newMeta.MetaDescription = metaDescription;
        //            metaInfoDict.Add(store.StoreID, newMeta);

        //        }
        //    }
        //    else
        //    {
        //        // Overwrite
        //        if (storeID == "0")
        //        {

        //            ProductMetaInformation meta = metaInfoDict["0"];
        //            meta.MetaKeyword = metaKeyword;
        //            meta.MetaDescription = metaDescription;

        //            //product.SetMetaKeyword("0", metaKeyword);
        //            //product.SetMetaDescription("0", metaDescription);
        //            foreach (Store store in _storeList)
        //            {

        //                if (!metaInfoDict.ContainsKey(store.StoreID))
        //                {
        //                    ProductMetaInformation newMeta = new ProductMetaInformation();
        //                    newMeta.StoreID = store.StoreID;
        //                    newMeta.MetaKeyword = metaKeyword;
        //                    newMeta.MetaDescription = metaDescription;
        //                    metaInfoDict.Add(store.StoreID, newMeta);

        //                }
        //                else
        //                {
        //                    meta = metaInfoDict[store.StoreID];
        //                    if (meta.UseDefaultMetaKeyword == true)
        //                    {
        //                        //product.SetMetaKeyword(store.StoreID, metaKeyword);

        //                        meta.MetaKeyword = metaKeyword;

        //                    }
        //                    if (meta.UseDefaultMetaDescription == true)
        //                    {
        //                        //product.SetMetaDescription(store.StoreID, metaDescription);
        //                        meta.MetaDescription = metaDescription;
        //                    }
        //                }

        //            }
        //        }
        //        else
        //        {
        //            product.SetUseDefaultValueMetaKeyword(storeID, false);
        //            product.SetUseDefaultValueMetaDescription(storeID, false);
        //            product.SetMetaKeyword(storeID, metaKeyword);
        //            product.SetMetaDescription(storeID, metaDescription);

        //            if (!metaInfoDict.ContainsKey(storeID))
        //            {
        //                ProductMetaInformation newMeta = new ProductMetaInformation();
        //                newMeta.StoreID = storeID;
        //                newMeta.MetaKeyword = metaKeyword;
        //                newMeta.MetaDescription = metaDescription;
        //                newMeta.UseDefaultMetaDescription = false;
        //                newMeta.UseDefaultMetaKeyword = false;
        //                metaInfoDict.Add(storeID, newMeta);
        //            }
        //            else
        //            {
        //                ProductMetaInformation meta = metaInfoDict[storeID];
        //                meta.MetaKeyword = metaKeyword;
        //                meta.MetaDescription = metaDescription;
        //                meta.UseDefaultMetaDescription = false;
        //                meta.UseDefaultMetaKeyword = false;
        //            }
        //        }
        //    }



        //    //MetaInformationCollection<ProductMetaInformation> metaList = new MetaInformationCollection<ProductMetaInformation>(new Product());

        //    //foreach (KeyValuePair<string, ProductMetaInformation> kvp in metaInfoDict)
        //    //{
        //    //    //ProductMetaInformation meta = new ProductMetaInformation();
        //    //    //meta.StoreID = kvp.Key;
        //    //    //meta.ProductID = product.ProductID;
        //    //    //meta.MetaKeyword = kvp.Value.MetaKeyword;
        //    //    //meta.MetaDescription = kvp.Value.MetaDescription;
        //    //    //meta.CultureID = kvp.Value.CultureID;
        //    //    //meta.



        //    //    metaList.Add(kvp.Value);
        //    //}

        //    //product.MetaInformations = metaList;



        //    //if (product.GetMetaInformation("0") == null)
        //    //{
        //    //    // Purge
        //    //    product.SetMetaKeyword("0", metaKeyword);
        //    //    product.SetMetaDescription("0", metaDescription);
        //    //    foreach (Store store in _storeList)
        //    //    {
        //    //        product.SetMetaKeyword(store.StoreID, metaKeyword);
        //    //        product.SetMetaDescription(store.StoreID, metaDescription);
        //    //    }
        //    //}
        //    //else
        //    //{
        //    //    // Overwrite
        //    //    if (storeID == "0")
        //    //    {
        //    //        product.SetMetaKeyword("0", metaKeyword);
        //    //        product.SetMetaDescription("0", metaDescription);
        //    //        foreach (Store store in _storeList)
        //    //        {
        //    //            ProductMetaInformation meta = product.GetMetaInformation(store.StoreID);
        //    //            if (meta == null)
        //    //            {
        //    //                product.SetMetaKeyword(store.StoreID, metaKeyword);
        //    //                product.SetMetaDescription(store.StoreID, metaDescription);
        //    //            }
        //    //            else
        //    //            {
        //    //                if (meta.UseDefaultMetaKeyword == true)
        //    //                {
        //    //                    product.SetMetaKeyword(store.StoreID, metaKeyword);
        //    //                }
        //    //                if (meta.UseDefaultMetaDescription == true)
        //    //                {
        //    //                    product.SetMetaDescription(store.StoreID, metaDescription);
        //    //                }
        //    //            }
        //    //        }
        //    //    }
        //    //    else
        //    //    {
        //    //        product.SetUseDefaultValueMetaKeyword(storeID, false);
        //    //        product.SetUseDefaultValueMetaDescription(storeID, false);
        //    //        product.SetMetaKeyword(storeID, metaKeyword);
        //    //        product.SetMetaDescription(storeID, metaDescription);
        //    //    }
        //    //}

        //    return product;
        //}



        //private Product SetProductPrice(Product product, string storeID, ProductPrice csvPrice)
        //{
        //    Dictionary<string, ProductPrice> priceDict = new Dictionary<string, ProductPrice>();

        //    foreach (ProductPrice price in product.ProductPrices)
        //    {
        //        priceDict.Add(price.StoreID, price);
        //    }

        //    if (!priceDict.ContainsKey(Store.Null.StoreID))
        //    {
        //        ProductPrice defaultPrice = new ProductPrice();
        //        defaultPrice.StoreID = Store.Null.StoreID;
        //        defaultPrice.Price = csvPrice.Price;
        //        defaultPrice.RetailPrice = csvPrice.RetailPrice;
        //        defaultPrice.WholesalePrice = csvPrice.WholesalePrice;
        //        defaultPrice.WholesalePrice2 = csvPrice.WholesalePrice2;
        //        defaultPrice.WholesalePrice3 = csvPrice.WholesalePrice3;
        //        defaultPrice = SetUseAllDefaultValue(defaultPrice);

        //        priceDict.Add(Store.Null.StoreID, defaultPrice);

        //        foreach (Store store in _storeList)
        //        {
        //            defaultPrice.StoreID = store.StoreID;
        //            priceDict.Add(store.StoreID, defaultPrice);
        //        }

        //    }
        //    else
        //    {
        //        ProductPrice productPrice = GetProductPrice(product, storeID, storeID, csvPrice, true, priceDict);

        //        if (storeID != "0")
        //        {
        //            //product.SetPrice(storeID, productPrice);

        //            if (priceDict.ContainsKey(storeID))
        //            {
        //                //priceDict[storeID].
        //                priceDict[storeID].Price = productPrice.Price;
        //                priceDict[storeID].RetailPrice = productPrice.RetailPrice;
        //                priceDict[storeID].WholesalePrice = productPrice.WholesalePrice;
        //                priceDict[storeID].WholesalePrice2 = productPrice.WholesalePrice2;
        //                priceDict[storeID].WholesalePrice3 = productPrice.WholesalePrice3;
        //                priceDict[storeID].UseDefaultPrice = productPrice.UseDefaultPrice;
        //                priceDict[storeID].UseDefaultRetailPrice = productPrice.UseDefaultRetailPrice;
        //                priceDict[storeID].UseDefaultWholeSalePrice = productPrice.UseDefaultWholeSalePrice;
        //                priceDict[storeID].UseDefaultWholeSalePrice2 = productPrice.UseDefaultWholeSalePrice2;
        //                priceDict[storeID].UseDefaultWholeSalePrice3 = productPrice.UseDefaultWholeSalePrice3;

        //                if (!KeyUtilities.IsMultistoreLicense())
        //                {
        //                    priceDict[storeID].UseDefaultPrice = true;
        //                    priceDict[storeID].UseDefaultRetailPrice = true;
        //                    priceDict[storeID].UseDefaultWholeSalePrice = true;
        //                    priceDict[storeID].UseDefaultWholeSalePrice2 = true;
        //                    priceDict[storeID].UseDefaultWholeSalePrice3 = true;
        //                }

        //            }
        //            else
        //            {
        //                productPrice.StoreID = storeID;
        //                priceDict.Add(storeID, productPrice);
        //            }

        //        }
        //        else
        //        {
        //            productPrice = SetUseAllDefaultValue(productPrice);
        //            //product.SetPrice(storeID, productPrice);

        //            if (priceDict.ContainsKey(storeID))
        //            {
        //                //priceDict[storeID].
        //                priceDict[storeID].Price = productPrice.Price;
        //                priceDict[storeID].RetailPrice = productPrice.RetailPrice;
        //                priceDict[storeID].WholesalePrice = productPrice.WholesalePrice;
        //                priceDict[storeID].WholesalePrice2 = productPrice.WholesalePrice2;
        //                priceDict[storeID].WholesalePrice3 = productPrice.WholesalePrice3;
        //                priceDict[storeID].UseDefaultPrice = productPrice.UseDefaultPrice;
        //                priceDict[storeID].UseDefaultRetailPrice = productPrice.UseDefaultRetailPrice;
        //                priceDict[storeID].UseDefaultWholeSalePrice = productPrice.UseDefaultWholeSalePrice;
        //                priceDict[storeID].UseDefaultWholeSalePrice2 = productPrice.UseDefaultWholeSalePrice2;
        //                priceDict[storeID].UseDefaultWholeSalePrice3 = productPrice.UseDefaultWholeSalePrice3;

        ////                if (!KeyUtilities.IsMultistoreLicense())
        ////                {
        ////                    priceDict[storeID].UseDefaultPrice = true;
        ////                    priceDict[storeID].UseDefaultRetailPrice = true;
        ////                    priceDict[storeID].UseDefaultWholeSalePrice = true;
        ////                    priceDict[storeID].UseDefaultWholeSalePrice2 = true;
        ////                    priceDict[storeID].UseDefaultWholeSalePrice3 = true;
        ////                }

        ////            }
        ////            else
        ////            {
        ////                productPrice.StoreID = storeID;
        ////                priceDict.Add(storeID, productPrice);
        ////            }

        ////            foreach (Store store in _storeList)
        ////            {
        ////                product.SetPrice(
        ////                    store.StoreID,
        ////                    GetProductPrice(product, store.StoreID, Store.Null.StoreID, csvPrice, false));

        ////                if (priceDict.ContainsKey(store.StoreID))
        ////                {
        ////                    //priceDict[storeID].
        ////                    priceDict[storeID].Price = productPrice.Price;
        ////                    priceDict[storeID].RetailPrice = productPrice.RetailPrice;
        ////                    priceDict[storeID].WholesalePrice = productPrice.WholesalePrice;
        ////                    priceDict[storeID].WholesalePrice2 = productPrice.WholesalePrice2;
        ////                    priceDict[storeID].WholesalePrice3 = productPrice.WholesalePrice3;
        ////                    priceDict[storeID].UseDefaultPrice = productPrice.UseDefaultPrice;
        ////                    priceDict[storeID].UseDefaultRetailPrice = productPrice.UseDefaultRetailPrice;
        ////                    priceDict[storeID].UseDefaultWholeSalePrice = productPrice.UseDefaultWholeSalePrice;
        ////                    priceDict[storeID].UseDefaultWholeSalePrice2 = productPrice.UseDefaultWholeSalePrice2;
        ////                    priceDict[storeID].UseDefaultWholeSalePrice3 = productPrice.UseDefaultWholeSalePrice3;

        ////                    if (!KeyUtilities.IsMultistoreLicense())
        ////                    {
        ////                        priceDict[storeID].UseDefaultPrice = true;
        ////                        priceDict[storeID].UseDefaultRetailPrice = true;
        ////                        priceDict[storeID].UseDefaultWholeSalePrice = true;
        ////                        priceDict[storeID].UseDefaultWholeSalePrice2 = true;
        ////                        priceDict[storeID].UseDefaultWholeSalePrice3 = true;
        ////                    }

        ////                }
        ////                else
        ////                {
        ////                    productPrice.StoreID = store.StoreID;
        ////                    priceDict.Add(store.StoreID, productPrice);
        ////                }
        ////            }
        ////        }
        ////    }
        ////    IList<ProductPrice> list = new List<ProductPrice>();

        ////    foreach (KeyValuePair<string, ProductPrice> kvp in priceDict)
        ////    {
        ////        list.Add(kvp.Value);
        ////    }

        ////    product.ProductPrices = list;

        ////    return product;
        ////}

        private Product SetProductPrice( Product product, string storeID, ProductPrice csvPrice )
        {
            //bool isNewProductPrice;
            //ProductPrice defaultPrice = product.GetProductPrice(Store.Null.StoreID, out isNewProductPrice);

            //if (isNewProductPrice)
            if (product.ProductID == "0")
            {
                ProductPrice defaultPrice = new ProductPrice();
                defaultPrice.Price = csvPrice.Price;
                defaultPrice.RetailPrice = csvPrice.RetailPrice;
                defaultPrice.WholesalePrice = csvPrice.WholesalePrice;
                defaultPrice.WholesalePrice2 = csvPrice.WholesalePrice2;
                defaultPrice.WholesalePrice3 = csvPrice.WholesalePrice3;
                defaultPrice = SetUseAllDefaultValue( defaultPrice );

                product.SetPrice( Store.Null.StoreID, defaultPrice );

                foreach (Store store in _storeList)
                {
                    product.SetPrice( store.StoreID, defaultPrice );
                }
            }
            else
            {
                ProductPrice productPrice = GetProductPrice( product, storeID, storeID, csvPrice, true );

                if (storeID != "0")
                {
                    product.SetPrice( storeID, productPrice );
                }
                else
                {
                    productPrice = SetUseAllDefaultValue( productPrice );
                    product.SetPrice( storeID, productPrice );

                    foreach (Store store in _storeList)
                    {
                        product.SetPrice(
                            store.StoreID,
                            GetProductPrice( product, store.StoreID, Store.Null.StoreID, csvPrice, false ) );
                    }
                }
            }
            return product;
        }

        private ProductPrice SetUseAllDefaultValue( ProductPrice productPrice )
        {
            productPrice.UseDefaultPrice = true;
            productPrice.UseDefaultRetailPrice = true;
            productPrice.UseDefaultWholeSalePrice = true;
            productPrice.UseDefaultWholeSalePrice2 = true;
            productPrice.UseDefaultWholeSalePrice3 = true;

            return productPrice;
        }

        private bool IsUseDefaultValue( bool isUpdateOneStore, bool enteredUseDefault )
        {
            if (isUpdateOneStore)
                return false;
            else
                return enteredUseDefault;
        }

        private ProductPrice GetProductPrice(
            Product product, string storeID, string storeViewID, ProductPrice csvPrice, bool isUpdateSelectedStore )
        {
            bool useDefault;

            ProductPrice productPrice = product.GetProductPrice( storeID );
            ProductPrice defaultPrice = product.GetProductPrice( Store.Null.StoreID );

            productPrice.Price = ProductPrice.VerifyPrice(
                storeID,
                storeViewID,
                csvPrice.Price,
                defaultPrice.Price,
                productPrice.Price,
                productPrice.UseDefaultPrice,
                IsUseDefaultValue( isUpdateSelectedStore, productPrice.UseDefaultPrice ),
                out useDefault );
            productPrice.UseDefaultPrice = useDefault;

            productPrice.RetailPrice = ProductPrice.VerifyPrice(
                storeID,
                storeViewID,
                csvPrice.RetailPrice,
                defaultPrice.RetailPrice,
                productPrice.RetailPrice,
                productPrice.UseDefaultRetailPrice,
                IsUseDefaultValue( isUpdateSelectedStore, productPrice.UseDefaultRetailPrice ),
                out useDefault );
            productPrice.UseDefaultRetailPrice = useDefault;

            productPrice.WholesalePrice = ProductPrice.VerifyPrice(
                storeID,
                storeViewID,
                csvPrice.WholesalePrice,
                defaultPrice.WholesalePrice,
                productPrice.WholesalePrice,
                productPrice.UseDefaultWholeSalePrice,
                IsUseDefaultValue( isUpdateSelectedStore, productPrice.UseDefaultWholeSalePrice ),
                out useDefault );
            productPrice.UseDefaultWholeSalePrice = useDefault;

            productPrice.WholesalePrice2 = ProductPrice.VerifyPrice(
               storeID,
               storeViewID,
               csvPrice.WholesalePrice2,
               defaultPrice.WholesalePrice2,
               productPrice.WholesalePrice2,
               productPrice.UseDefaultWholeSalePrice2,
               IsUseDefaultValue( isUpdateSelectedStore, productPrice.UseDefaultWholeSalePrice2 ),
               out useDefault );
            productPrice.UseDefaultWholeSalePrice2 = useDefault;

            productPrice.WholesalePrice3 = ProductPrice.VerifyPrice(
              storeID,
              storeViewID,
              csvPrice.WholesalePrice3,
              defaultPrice.WholesalePrice3,
              productPrice.WholesalePrice3,
              productPrice.UseDefaultWholeSalePrice3,
              IsUseDefaultValue( isUpdateSelectedStore, productPrice.UseDefaultWholeSalePrice3 ),
              out useDefault );
            productPrice.UseDefaultWholeSalePrice3 = useDefault;

            return productPrice;
        }

        private ProductPrice GetProductPriceInDict( string key, Dictionary<string, ProductPrice> priceDict )
        {
            if (priceDict.ContainsKey( key ))
                return priceDict[key];
            else
                return new ProductPrice();
        }

        private ProductPrice GetProductPrice(
            Product product, string storeID, string storeViewID, ProductPrice csvPrice,
            bool isUpdateSelectedStore, Dictionary<string, ProductPrice> priceDict )
        {
            bool useDefault;

            ProductPrice productPrice = GetProductPriceInDict( storeID, priceDict );
            productPrice.Price = ProductPrice.VerifyPrice(
                storeID,
                storeViewID,
                csvPrice.Price,
                GetProductPriceInDict( Store.Null.StoreID, priceDict ).Price,
                productPrice.Price,
                productPrice.UseDefaultPrice,
                IsUseDefaultValue( isUpdateSelectedStore, productPrice.UseDefaultPrice ),
                out useDefault );
            productPrice.UseDefaultPrice = useDefault;

            productPrice.RetailPrice = ProductPrice.VerifyPrice(
                storeID,
                storeViewID,
                csvPrice.RetailPrice,
                GetProductPriceInDict( Store.Null.StoreID, priceDict ).RetailPrice,
                productPrice.RetailPrice,
                productPrice.UseDefaultRetailPrice,
                IsUseDefaultValue( isUpdateSelectedStore, productPrice.UseDefaultRetailPrice ),
                out useDefault );
            productPrice.UseDefaultRetailPrice = useDefault;

            productPrice.WholesalePrice = ProductPrice.VerifyPrice(
                storeID,
                storeViewID,
                csvPrice.WholesalePrice,
                GetProductPriceInDict( Store.Null.StoreID, priceDict ).WholesalePrice,
                productPrice.WholesalePrice,
                productPrice.UseDefaultWholeSalePrice,
                IsUseDefaultValue( isUpdateSelectedStore, productPrice.UseDefaultWholeSalePrice ),
                out useDefault );
            productPrice.UseDefaultWholeSalePrice = useDefault;

            productPrice.WholesalePrice2 = ProductPrice.VerifyPrice(
               storeID,
               storeViewID,
               csvPrice.WholesalePrice2,
               GetProductPriceInDict( Store.Null.StoreID, priceDict ).WholesalePrice2,
               productPrice.WholesalePrice2,
               productPrice.UseDefaultWholeSalePrice2,
               IsUseDefaultValue( isUpdateSelectedStore, productPrice.UseDefaultWholeSalePrice2 ),
               out useDefault );
            productPrice.UseDefaultWholeSalePrice2 = useDefault;

            productPrice.WholesalePrice3 = ProductPrice.VerifyPrice(
              storeID,
              storeViewID,
              csvPrice.WholesalePrice3,
              GetProductPriceInDict( Store.Null.StoreID, priceDict ).WholesalePrice3,
              productPrice.WholesalePrice3,
              productPrice.UseDefaultWholeSalePrice3,
              IsUseDefaultValue( isUpdateSelectedStore, productPrice.UseDefaultWholeSalePrice3 ),
              out useDefault );
            productPrice.UseDefaultWholeSalePrice3 = useDefault;

            return productPrice;
        }

        private ArrayList GetTaxClassIDList()
        {
            ArrayList result = new ArrayList();


            IList<TaxClass> list = DataAccessContext.TaxClassRepository.GetAll( "TaxClassID" );
            foreach (TaxClass item in list)
            {
                string id = item.TaxClassID;
                result.Add( id );
            }

            return result;
        }

        private ArrayList GetCategoryVisibleIDList()
        {
            ArrayList result = new ArrayList();
            foreach (DataRow row in _allCategoriesListTable.Rows)
            {
                string id = row["CategoryID"].ToString();

                if (CheckParentCategoryVisibility( id ))
                    result.Add( id );

            }

            return result;

        }

        private bool CheckParentCategoryVisibility( string categoryID )
        {
            if (categoryID == "0")
                return true;

            //Category category = DataAccessContext.CategoryRepository.GetOne(Culture.Null, categoryID);

            DataTable categoryTable = DataAccess.ExecuteSelect(
               "SELECT * " + "FROM Category " + "WHERE Category.CategoryID = @CategoryID; ",
               DataAccess.CreateParameterString( categoryID ) );


            // if (category == null)
            if (categoryTable.Rows.Count == 0)
            {
                return false;
            }
            //else if (!category.IsEnabled)
            else if (!ConvertUtilities.ToBoolean( categoryTable.Rows[0]["IsEnabled"].ToString() ))
            {
                return false;
            }
            // else if (category.ParentCategoryID == "0")
            else if (categoryTable.Rows[0]["ParentCategoryID"].ToString() == "0")
            {
                return ConvertUtilities.ToBoolean( categoryTable.Rows[0]["IsEnabled"].ToString() );
            }
            else
            {
                return CheckParentCategoryVisibility( categoryTable.Rows[0]["ParentCategoryID"].ToString() );
            }
        }

        private bool AddProductImage( Product product, CsvReader csv, bool isSkipImage )
        {
            int currentimage;
            bool hasSecondary = false;
            for (currentimage = _largeImage1; currentimage <= _largeImage10; currentimage = currentimage + 3)
            {
                if (!string.IsNullOrEmpty( csv[currentimage] ))
                {
                    //string largeFilePath = ProductImageFile.CopyToLargeImageFolder(_fileManager, csv[currentimage].ToString());
                    string largeFilePath;
                    if (isSkipImage)
                    {
                        if (!ProductImageFile.IsImageInLargeDirectory( _fileManager, Path.GetFileName( csv[currentimage].ToString() ) ))
                        {
                            largeFilePath = ProductImageFile.CopyToLargeImageFolder( _fileManager, csv[currentimage].ToString() );
                        }
                        else
                        {
                            largeFilePath = ProductImageFile.LargeRelativeDirectory + Path.GetFileName( csv[currentimage].ToString() );
                        }
                    }
                    else
                    {
                        largeFilePath = ProductImageFile.CopyToLargeImageFolder( _fileManager, csv[currentimage].ToString() );
                    }

                    if (SaveRegularAndThumbnail( largeFilePath, product, currentimage, csv, isSkipImage ))
                    {
                        if (hasSecondary == false)
                        {
                            if (SaveSecondaryImage( largeFilePath, product ))
                                hasSecondary = true;
                            else
                                return false;
                        }
                    }
                    else
                        return false;
                }
            }
            return true;
        }

        private Image GetImageFromLocal( string imageFilePath )
        {
            try
            {
                string path = _fileManager.GetFullPath( GetPathWithTilde( imageFilePath ) );
                using (FileStream fs = new FileStream( path, FileMode.Open, FileAccess.Read ))
                {
                    return Image.FromStream( fs );
                }
            }
            catch
            {
                return null;
            }
        }

        private bool SaveSecondaryImage( string largeFilePath, Product product )
        {
            System.Drawing.Image myImage = GetImageFromLocal( largeFilePath );

            if (myImage == null)
            {
                _status.AddImageProcessError( largeFilePath );
                return false;
            }

            try
            {
                using (ProductImageFile imageFile =
                    new ProductImageFile( _fileManager, Path.GetFileName( largeFilePath ), myImage ))
                {
                    imageFile.SaveSecondary();
                    product.ImageSecondary = imageFile.SecondaryFilePath;
                }
            }
            finally
            {
                myImage.Dispose();
            }
            return true;
        }

        private string GetPathWithTilde( string relativePath )
        {
            if (relativePath.Trim().StartsWith( "~/" ))
                return relativePath.Trim();
            else
                return "~/" + relativePath.Trim();
        }

        private bool IsImageExists( CsvReader csv )
        {
            for (int currentimage = _largeImage1; currentimage <= _largeImage10; currentimage = currentimage + 3)
            {
                if (!string.IsNullOrEmpty( csv[currentimage] ))
                {
                    if (!_fileManager.Exists( GetPathWithTilde( csv[currentimage] ) ))
                    {
                        return false;
                    }
                }
            }

            return true;
        }

        private bool SaveRegularAndThumbnail(
            string largeFilePath, Product product, int currentimage, CsvReader csv, bool isSkipImage )
        {
            System.Drawing.Image myImage = GetImageFromLocal( largeFilePath );
            if (myImage == null)
            {
                _status.AddImageProcessError( largeFilePath );
                return false;
            }

            try
            {
                using (ProductImageFile imageFile = new ProductImageFile( _fileManager, Path.GetFileName( largeFilePath ), myImage ))
                {
                    //imageFile.SaveRegular();
                    //imageFile.SaveThumbnail();
                    if (isSkipImage)
                    {
                        if (!ProductImageFile.IsImageInRegularDirectory( _fileManager, Path.GetFileName( largeFilePath ) ))
                        {
                            imageFile.SaveRegular();
                        }
                        if (!ProductImageFile.IsImageInThumbnailDirectory( _fileManager, Path.GetFileName( largeFilePath ) ))
                        {
                            imageFile.SaveThumbnail();
                        }

                    }
                    else
                    {
                        imageFile.SaveRegular();
                        imageFile.SaveThumbnail();
                    }

                    bool isZoom = false;
                    bool isEnlarge = false;
                    if (csv[currentimage + 1] != null)
                    {
                        isZoom = ConvertToBool( csv[currentimage + 1] );
                    }
                    if (csv[currentimage + 2] != null)
                    {
                        isEnlarge = ConvertToBool( csv[currentimage + 2] );
                    }
                    ProductImage productImage = new ProductImage();
                    productImage.RegularImage = imageFile.RegularFilePath;
                    productImage.LargeImage = largeFilePath;
                    productImage.ThumbnailImage = imageFile.ThumbnailFilePath;
                    productImage.IsZoom = isZoom;
                    productImage.IsEnlarge = isEnlarge;
                    productImage.SortOrder = product.ProductImages.Count;

                    product.ProductImages.Add( productImage );
                }
            }
            finally
            {
                myImage.Dispose();
            }
            return true;
        }


        private bool OverwriteProduceItem( CsvReader csv, bool processImage, bool isSkipImage, string storeID )
        {
            bool result = false;
            string sku = csv[_skuIndex].ToString();

            Product product = GetProductBySku( sku, storeID, ConvertToBool( csv[_useOptionIndex] ) );

            result = SaveProductItem( product, csv, processImage, isSkipImage, storeID );

            _importedSkuList.Add( sku );

            return result;
        }

        private Product GetProductBySku( string sku, string storeID, bool isUseOption )
        {
            string productID = GetProductIDBySku( sku, isUseOption );

            //if (!String.IsNullOrEmpty(productID))
            //    return DataAccessContext.ProductRepository.GetOneLargeProduct(_culture, productID, storeID);
            //else
            //    return new Product(_culture);


            if (!String.IsNullOrEmpty( productID ))
            {
                return DataAccessContext.ProductRepository.GetOneLargeProduct( _culture, productID, storeID );
            }
            else
            {
                if (isUseOption)
                {
                    Product product = DataAccessContext.ProductRepository.GetOneBySku( _culture, _selectedStoreID, sku );
                    if (product != Product.Null)
                    {
                        return product;
                    }
                }
            }
            return new Product( _culture );
        }

        private string GetProductIDBySku( string sku, bool isUseOption )
        {
            if (_skuDatabase.ContainsKey( sku ))
            {
                return _skuDatabase[sku];
                //return String.Empty;
            }

            //if (isUseOption)
            //{
            //    Product product = DataAccessContext.ProductRepository.GetOneBySku(_culture, _selectedStoreID, sku);

            //    if (product != Product.Null)
            //    {
            //        return product.ProductID;
            //    }
            //}

            // else
            //  {
            return String.Empty;
            // }

        }

        private bool SaveProductItem( Product product, CsvReader csv, bool processImage, bool isSkipImage, string storeID )
        {
            string sku = csv[_skuIndex].ToString();

            double merchantRating = product.MerchantRating;
            string discountGroupID = String.IsNullOrEmpty( product.DiscountGroupID )
                ? "0" : product.DiscountGroupID;

            product.Name = csv[_nameIndex];
            product.ShortDescription = csv[_shortDescriptionIndex];
            product.LongDescription = csv[_longDescriptionIndex];
            product.Sku = sku;
            product.Model = csv[_modelIndex];
            product.Weight = ConvertUtilities.ToDouble( (csv[_weightIndex].ToString()) );
            //   product.FixedShippingCost = false;
            //   product.ShippingCost = ConvertUtilities.ToDecimal("0");
            product.Manufacturer = csv[_manufacturerIndex];
            product.Keywords = csv[_keywordsIndex];
            product.Other1 = csv[_other1Index];
            product.Other2 = csv[_other2Index];
            product.Other3 = csv[_other3Index];
            product.Other4 = csv[_other4Index];
            product.Other5 = csv[_other5Index];
            product.IsTodaySpecial = ConvertToBool( csv[_isTodaySpecialIndex] );
            product.IsDownloadable = ConvertToBool( csv[_isDownloadableIndex] );
            product.DownloadPath = csv[_downloadPathIndex];
            product.IsEnabled = ConvertToBool( csv[_isEnabledIndex] );
            product.RelatedProducts = csv[_relatedProducts];
            product.MerchantRating = merchantRating;
            product.DiscountGroupID = discountGroupID;
            product.IsFixedPrice = true;
            product.ManufacturerPartNumber = csv[_manufacturerPartNumber];
            product.Upc = csv[_upc];
            product.TaxClassID = ConvertToTaxClassID( csv[_taxClassID] );
            product.UseInventory = ConvertToBool( csv[_useInventory] );
            product.IsRecurring = ConvertToBool( csv[_isRecurring] );
            product.IsAffiliate = ConvertToBool( csv[_isAffiliate] );
            product.FreeShippingCost = ConvertToBool( csv[_isFreeShippingCost] );

            product.CategoryIDs = ConvertToCategoryIDs( csv[_categoryIDIndex] );
            product.DepartmentIDs = ConvertToCategoryIDs( csv[_departmentIDIndex] );

            product = SetProductMetaInformation( product, storeID, csv[_metaKeyword], csv[_metaDescription] );

            ProductPrice csvPrice = new ProductPrice();
            csvPrice.Price = ConvertUtilities.ToDecimal( (csv[_priceIndex]).ToString() );
            csvPrice.RetailPrice = ConvertUtilities.ToDecimal( (csv[_retailPriceIndex]).ToString() );
            csvPrice.WholesalePrice = ConvertUtilities.ToDecimal( (csv[_wholesalePrice1Index]).ToString() );
            csvPrice.WholesalePrice2 = ConvertUtilities.ToDecimal( (csv[_wholesalePrice2Index]).ToString() );
            csvPrice.WholesalePrice3 = ConvertUtilities.ToDecimal( (csv[_wholesalePrice3Index]).ToString() );
            product = SetProductPrice( product, storeID, csvPrice );

            // product.ProductShippingCosts.Clear();

            if (ConvertToBool( csv[_useOptionIndex] ))
            {
                CreateOptionAndStock( product, csv );
            }
            else
            {
                //product.SetStock(ConvertUtilities.ToInt32((csv[_stockIndex]).ToString()));
                product.SetStockWithNoOption( ConvertUtilities.ToInt32( (csv[_stockIndex]).ToString() ) );
            }

            if (processImage)
            {
                try
                {
                    product.ProductImages.Clear();
                    if (!AddProductImage( product, csv, isSkipImage ))
                    {
                        return false;
                    }
                }
                catch (Exception)
                {
                    throw;
                }
            }

            if (product.IsRecurring)
            {
                ProductRecurring recurring = new ProductRecurring();
                recurring.RecurringInterval = ConvertUtilities.ToInt32( csv[_recurringInterval] );
                recurring.RecurringIntervalUnit = Convert.ToString( csv[_recurringIntervalUnit] );
                recurring.RecurringNumberOfCycles = Convert.ToInt32( csv[_recurringNumberOfCycles] );
                recurring.RecurringNumberOfTrialCycles = Convert.ToInt32( csv[_recurringNumberOfTrialCycles] );
                recurring.RecurringTrialAmount = ConvertUtilities.ToDecimal( csv[_recurringTrialAmount] );
                product.ProductRecurring = recurring;
            }



            product = SaveProduct( product, csv );
            //  DataAccessContext.ProductRepository.RefreshProductUrlNameAll(product.Name);

            if (!String.IsNullOrEmpty( product.RelatedProducts ))
                _updateRelatedIDList.Add( product.ProductID, product.RelatedProducts );

            return true;
        }

        private Product SaveProduct( Product product, CsvReader csv )
        {
            Product result;

            if (product.ProductID == "0")
                result = DataAccessContext.ProductRepository.CreateLargeProductItem(
                    product, GetLocale( csv ), _categoryVisibleIDList, _processImage, ConvertToBool( csv[_useOptionIndex] ) );

            else
                result = DataAccessContext.ProductRepository.UpdateLargeProductItem(
                    product, _categoryVisibleIDList, _processImage, ConvertToBool( csv[_useOptionIndex] ) );

            return result;
        }

        private ProductLocale GetLocale( CsvReader csv )
        {
            ProductLocale locale = new ProductLocale();
            locale.CultureID = _culture.CultureID;
            locale.Name = csv[_nameIndex];
            locale.ShortDescription = csv[_shortDescriptionIndex];
            locale.LongDescription = csv[_longDescriptionIndex];

            return locale;
        }


        private bool IsHasOptionItemInList( IList<OptionItem> optionItemLists, string optionItemID )
        {
            foreach (OptionItem item in optionItemLists)
            {
                if (item.OptionItemID == optionItemID)
                    return true;
            }
            return false;
        }

        private bool CheckOptions( CsvReader csv )
        {
            if (ConvertToBool( csv[_useOptionIndex] ))
            {
                int optionCount = 1;
                for (int currentOption = _optionGroup1IDIndex; currentOption <= _optionGroup5IDIndex; currentOption = currentOption + 2)
                {
                    if (!String.IsNullOrEmpty( csv[currentOption] ))
                    {
                        IList<OptionItem> itemLists = DataAccessContext.OptionItemRepository.GetByOptionGroupID( _culture, csv[currentOption] );
                        if (itemLists.Count > 0)
                        {
                            if (ConvertToBool( csv[currentOption + 1] ))
                            {
                                if (!ConvertToBool( csv[_useInventory] ))
                                    return false;
                                if (!IsHasOptionItemInList( itemLists, String.Format( "{0}", csv[currentOption + (11 - optionCount)] ) ))
                                    return false;
                            }
                            else
                            {
                                if (!String.IsNullOrEmpty( csv[currentOption + (11 - optionCount)] ))
                                    return false;
                            }
                        }
                        else
                            return false;
                    }
                    optionCount++;
                }
            }
            return true;
        }

        private bool CheckRecurringGiftDownload( CsvReader csv )
        {
            if (ConvertToBool( csv[_isRecurring] ))
            {
                if (ConvertToBool( csv[_isGiftCertificate] ) || ConvertToBool( csv[_isDownloadableIndex] ))
                    return false;
            }
            return true;
        }

        private bool CheckRecurringIntervalCycles( CsvReader csv )
        {
            if (ConvertToBool( csv[_isRecurring] ))
            {
                if (ConvertUtilities.ToInt32( csv[_recurringInterval] ) <= 0)
                    return false;
                else if (ConvertUtilities.ToInt32( csv[_recurringNumberOfCycles] ) <= 0)
                    return false;
            }
            return true;
        }

        private bool CheckRecurringIntervalUnit( CsvReader csv )
        {
            if (ConvertToBool( csv[_isRecurring] ))
            {
                string intervalUnit = csv[_recurringIntervalUnit];
                switch (intervalUnit)
                {
                    case "Day":
                        return true;
                    case "Week":
                        return true;
                    case "Month":
                        return true;
                    case "Year":
                        return true;
                    default:
                        return false;
                }
            }
            return true;
        }

        private void CreateOptionGroup( Product product, string optionGroupID, string useStock )
        {
            if (!String.IsNullOrEmpty( optionGroupID ))
            {
                OptionGroup optionGroup = DataAccessContext.OptionGroupRepository.GetOne(
                    _culture, optionGroupID );

                product.AddOptionGroup( optionGroup, ConvertToBool( useStock ) );
            }
        }

        private void CreateStock( Product product, string optionItemID, string stockItem, bool stockEnabled )
        {
            if (!String.IsNullOrEmpty( optionItemID ) && !String.IsNullOrEmpty( stockItem ))
            {
                string[] optionItemIDs =
                    optionItemID.Split( new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries );

                product.SetStock( ConvertUtilities.ToInt32( stockItem ), optionItemIDs );
            }
            else
            {
                product.SetStock( ConvertUtilities.ToInt32( stockItem ) );
            }
        }

        private string GetFormatOptionItemID( CsvReader csv )
        {
            return String.Format( "{0} {1} {2} {3} {4}",
                csv[_optionItem1IDIndex],
                csv[_optionItem2IDIndex],
                csv[_optionItem3IDIndex],
                csv[_optionItem4IDIndex],
                csv[_optionItem5IDIndex] ).Trim();
        }

        private bool IsOptionGroupValid( string optionGroupID, string useStock, Product product )
        {
            if (String.IsNullOrEmpty( optionGroupID ))
                return true;

            foreach (ProductOptionGroup productOptionGroup in product.ProductOptionGroups)
            {
                if (productOptionGroup.OptionGroupID == optionGroupID
                    && productOptionGroup.UseStock == ConvertToBool( useStock ))
                {
                    return true;
                }
            }
            return false;
        }

        private void AssertOptionGroupsAreSame( Product product, CsvReader csv )
        {
            if (!IsOptionGroupValid( csv[_optionGroup1IDIndex], csv[_useStock1Index], product )
                || !IsOptionGroupValid( csv[_optionGroup2IDIndex], csv[_useStock2Index], product )
                || !IsOptionGroupValid( csv[_optionGroup3IDIndex], csv[_useStock3Index], product )
                || !IsOptionGroupValid( csv[_optionGroup4IDIndex], csv[_useStock4Index], product )
                || !IsOptionGroupValid( csv[_optionGroup5IDIndex], csv[_useStock5Index], product ))
            {
                throw new DomainException(
                    "OptionGroup[X]ID or UseStock[X]Index does not match with values " +
                    "of the same product on another row" );
            }
        }

        private void CreateOptionAndStock( Product product, CsvReader csv )
        {
            if (product.ProductOptionGroups.Count == 0)
            {
                CreateOptionGroup( product, csv[_optionGroup1IDIndex], csv[_useStock1Index] );
                CreateOptionGroup( product, csv[_optionGroup2IDIndex], csv[_useStock2Index] );
                CreateOptionGroup( product, csv[_optionGroup3IDIndex], csv[_useStock3Index] );
                CreateOptionGroup( product, csv[_optionGroup4IDIndex], csv[_useStock4Index] );
                CreateOptionGroup( product, csv[_optionGroup5IDIndex], csv[_useStock5Index] );
            }
            else
            {
                AssertOptionGroupsAreSame( product, csv );
            }

            CreateStock(
                product,
                GetFormatOptionItemID( csv ),
                (csv[_stockIndex].ToString()),
                true );
        }

        private void ParseErrorHandler( object sender, ParseErrorEventArgs e )
        {
            _status.AddError();
        }

        private CsvReader CreateCsvReader( string relativePath, out TextReader reader )
        {
            Encoding encoding;
            if (_fileManager.ContainsUnicodeBom( relativePath ))
                encoding = Encoding.Unicode;
            else
                encoding = Encoding.Default;

            reader = _fileManager.CreateTextReader( relativePath, encoding );
            CsvReader csv = new CsvReader( reader, true );
            csv.MissingFieldAction = MissingFieldAction.ReplaceByEmpty;
            csv.ParseError += ParseErrorHandler;

            return csv;
        }

        private ArrayList GetValidDepartmentIDList()
        {
            ArrayList result = new ArrayList();

            DataTable table = DataAccess.ExecuteSelect(
                "SELECT DepartmentID, RootID FROM Department WHERE RootID != 0 ; " );

            if (table.Rows.Count > 0)
            {
                foreach (DataRow row in table.Rows)
                {
                    string id = row["DepartmentID"].ToString();

                    if (IsLeafDepartment( id ))
                        result.Add( id );
                }
            }

            return result;
        }

        private bool IsLeafDepartment( string departmentID )
        {
            string count = DataAccess.ExecuteScalar(
                "SELECT COUNT(*) " + "FROM Department " +
                "WHERE ParentDepartmentID = @ParentDepartmentID; ",
                DataAccess.CreateParameterString( departmentID ) );

            return int.Parse( count ) == 0;
        }

        private ArrayList GetValidCategoryIDList()
        {
            ArrayList result = new ArrayList();

            _allCategoriesListTable = DataAccess.ExecuteSelect(
                "SELECT CategoryID, RootID FROM Category WHERE RootID != 0 ; " );

            if (_allCategoriesListTable.Rows.Count > 0)
            {
                foreach (DataRow row in _allCategoriesListTable.Rows)
                {
                    string id = row["CategoryID"].ToString();

                    if (IsLeafCategory( id ))
                        result.Add( id );
                }
            }

            return result;

        }

        private bool IsLeafCategory( string categoryID )
        {
            string count = DataAccess.ExecuteScalar(
                "SELECT COUNT(*) " + "FROM Category " +
                "WHERE ParentCategoryID = @ParentCategoryID; ",
                DataAccess.CreateParameterString( categoryID ) );

            return int.Parse( count ) == 0;
        }

        private Dictionary<string, string> GetProductIDAndSku()
        {
            Dictionary<string, string> skuProductID = new Dictionary<string, string>();

            DataTable table = DataAccess.ExecuteSelect( "Select ProductID, Sku from Product" );
            foreach (DataRow row in table.Rows)
            {
                if (!skuProductID.ContainsKey( row["Sku"].ToString() ))
                {
                    skuProductID.Add( row["Sku"].ToString(), row["ProductID"].ToString() );
                }
            }
            return skuProductID;
        }


        private string[] GetArrayOfRelatedProducts( string relatedProducts )
        {
            return relatedProducts.Split( '|' );
        }

        public string GetRelatedProductsID( ArrayList relatedArray )
        {
            int i = 0;
            int count = relatedArray.Count;
            string relatedProductIDs = String.Empty;

            if (relatedArray == null)
                return relatedProductIDs;
            else
            {
                foreach (Object productID in relatedArray)
                {
                    if (i != (count - 1))
                        relatedProductIDs += (productID.ToString() + ", ");
                    else
                        relatedProductIDs += productID.ToString();
                    i++;
                }
                return relatedProductIDs;
            }
        }

        private void UpdateRelatedProduct( string productID, string relatedID )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery( "UPDATE Product SET RelatedProducts = @RelatedProducts WHERE ProductID = @ProductID ; ",
                    DataAccess.CreateParameterString( relatedID ),
                    DataAccess.CreateParameterString( productID ) );

                transaction.Complete();
            }
        }

        private void UpdateRelatedProductsFromSkuToID()
        {
            foreach (KeyValuePair<string, string> kvp in _updateRelatedIDList)
            {
                ArrayList relatedArray = new ArrayList();

                if (!String.IsNullOrEmpty( kvp.Value ))
                {
                    string[] relatedProductSku = GetArrayOfRelatedProducts( kvp.Value );

                    for (int i = 0; i < relatedProductSku.Length; i++)
                    {
                        string relatedID = GetRelatedProductIDBySku( relatedProductSku[i].Trim() );
                        if (!String.IsNullOrEmpty( relatedID ) && !relatedArray.Contains( relatedID ))
                            relatedArray.Add( relatedID );
                    }

                    if (relatedArray.Count > 0)
                        UpdateRelatedProduct( kvp.Key, GetRelatedProductsID( relatedArray ) );
                }
            }
        }

        private ProductImportStatus ImportProduct(
            string filePath, bool processImage, bool isSkipImage, out TimeSpan timeDiff, string storeID
            )
        {
            TextReader reader = null;
            CsvReader csv = null;
            _status = new ProductImportStatus();

            try
            {
                //     LoadSkuFromCsv(filePath);
                csv = CreateCsvReader( filePath, out reader );
                // Read data line-by-line

                //csv.First();
                int i = 2;
                while (csv.ReadNextRecord())
                {
                    try
                    {
                        if (!IsCategoryExist( csv[_categoryIDIndex] ))
                        {
                            _status.AddCategoryError( i.ToString() );
                        }
                        //else if (!IsRelatedProductExists(csv[_relatedProducts]))
                        //{
                        //    _status.AddRelatedProductError(i.ToString());
                        //}
                        else if (!IsDepartmentExist( csv[_departmentIDIndex] ))
                        {
                            _status.AddDepartmentError( i.ToString() );
                        }
                        else if (processImage && !IsImageExists( csv ))
                        {
                            _status.AddImageUploadError( i.ToString() );
                        }
                        else if (!CheckOptions( csv ))
                        {
                            _status.AddUseOptionError( i.ToString() );
                        }
                        else if (!CheckRecurringGiftDownload( csv ))
                        {
                            _status.AddRecurringGiftDownloadError( i.ToString() );
                        }
                        else if (!CheckRecurringIntervalCycles( csv ))
                        {
                            _status.AddRecurringIntervalCyclesError( i.ToString() );
                        }
                        else if (!CheckRecurringIntervalUnit( csv ))
                        {
                            _status.AddRecurringIntervalUnitError( i.ToString() );
                        }
                        else if (!IsTaxClassExist( csv[_taxClassID] ))
                        {
                            _status.AddTaxClassError( i.ToString() );
                        }
                        else
                        {
                            if (OverwriteProduceItem( csv, processImage, isSkipImage, storeID ))
                                _status.AddSuccess();
                        }
                    }
                    catch (Exception ex)
                    {
                        _status.AddOtherError( i.ToString(), ex );
                    }
                    i++;
                }

                try
                {
                    UpdateRelatedProductsFromSkuToID();
                }
                catch (Exception ex)
                {
                    _status.SetFatalError( "Related Product - " + ex.Message );
                }

                _status.BuildRelatedProductError();
                _status.BuildCategoryError();
                _status.BuildDepartmentError();
                _status.BuildImageUploadError();
                _status.BuildOtherError();
                _status.BuildUseOptionError();
                _status.BuildImageProcessError();
                _status.BuildRecurringGiftDownloadError();
                _status.BuildRecurringIntervalCyclesError();
                _status.BuildRecurringIntervalUnitError();
                _status.BuildTaxClassError();
            }
            catch (Exception ex)
            {
                _status.SetFatalError( ex.Message );
            }
            finally
            {
                if (csv != null)
                    csv.Dispose();

                if (reader != null)
                    reader.Dispose();
            }

            DataAccessContext.ProductRepository.RefreshProductUrlNameAll();
            _timeEnd = DateTime.Now;
            timeDiff = _timeEnd.Subtract( _timeStart );

            return _status;
        }




        #endregion


        #region Constructors



        public LargeImporter( Culture culture, IFileManager fileManager )
        {
            _culture = culture;
            _fileManager = fileManager;

            _storeList = DataAccessContext.StoreRepository.GetAll( "StoreID" );
            _taxClassIDList = GetTaxClassIDList();
            _categoryIDList = GetValidCategoryIDList();
            _firstTaxClassID = DataAccess.ExecuteScalar(
                "SELECT TOP 1 TaxClassID FROM TaxClass ORDER BY TaxClassID ASC; " ).ToString();
            _categoryVisibleIDList = GetCategoryVisibleIDList();

            _departmentIDList = GetValidDepartmentIDList();
        }


        public LargeImporter(
            Culture culture,
            IFileManager fileManager,
            ArrayList validCategoryIDList,
            ArrayList categoryVisibleIDList,
            ArrayList validDepartmentIDList,
            string firstTaxClassID )
        {
            _culture = culture;
            _fileManager = fileManager;

            _storeList = DataAccessContext.StoreRepository.GetAll( "StoreID" );
            _taxClassIDList = GetTaxClassIDList();

            _categoryIDList = validCategoryIDList;
            _categoryVisibleIDList = categoryVisibleIDList;
            _firstTaxClassID = firstTaxClassID;

            _departmentIDList = validDepartmentIDList;
        }

        #endregion

        #region Public Methods
        private bool _processImage;


        public ProductImportStatus ImportProductPurgeAll(
            string filePath, bool processImage, bool isSkipImage, out TimeSpan timeDiff, string storeID )
        {
            _processImage = processImage;
            _timeStart = DateTime.Now;
            DataAccessContext.ProductRepository.DeleteAll();
            _selectedStoreID = storeID;

            return ImportProduct( filePath, processImage, isSkipImage, out timeDiff, storeID );
        }

        public ProductImportStatus ImportProductOverwrite(
            string filePath,
            bool processImage,
             bool isSkipImage,
            out TimeSpan timeDiff,
            string storeID,
            Dictionary<string, string> productIDAndSku )
        {
            _processImage = processImage;
            _timeStart = DateTime.Now;
            _skuDatabase = productIDAndSku;
            _selectedStoreID = storeID;

            return ImportProduct( filePath, processImage, isSkipImage, out timeDiff, storeID );
        }

        #endregion

    }
}
