﻿using System.Collections.Specialized;
using Vevo.Domain.Orders;
using Vevo.Domain.Users;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.Payments.TwoCheckout
{
    public class TwoCheckoutRedirectPaymentMethod : RedirectPaymentMethod
    {
        public override string GetPostedUrl()
        {
            return "https://www.2checkout.com/2co/buyer/purchase";
        }

        public override NameValueCollection CreatePostParameters(
           CheckoutDetails checkoutDetails,
           Culture culture,
           Currency currency,
           Cart shoppingCart,
           string storefrontUrl,
           string orderID,
           decimal orderTotal,
           WholesaleStatus wholesaleStatus,
           string ipAddress )
        {

            NameValueCollection collection = new NameValueCollection();

            collection.Add( "sid", DataAccessContext.Configurations.GetValue( "2COMerchantAccount" ) );
            collection.Add( "total", Currency.ConvertPriceToUSFormat( orderTotal ) );
            collection.Add( "cart_order_id", orderID );
            collection.Add( "merchant_order_id", orderID );
            collection.Add( "fixed", "Y" );
            collection.Add( "pay_method", "CC" );
            collection.Add( "id_type", "1" );

            int i = 1;

            foreach (ICartItem item in shoppingCart.CartItems)
            {
                collection.Add( "c_prod_" + i.ToString(), item.ProductID + "," + item.Quantity );
                collection.Add( "c_name_" + i.ToString(), item.Product.Name );
                collection.Add( "c_description_" + i.ToString(), "N/A" );
                collection.Add( "c_price_" + i.ToString(), Currency.ConvertPriceToUSFormat( item.GetCheckoutUnitPrice( wholesaleStatus ) ) );
                collection.Add( "c_tangible_" + i.ToString(), ConvertUtilities.ToYN( item.IsDownloadable ) );
                i++;
            }

            collection.Add( "card_holder_name", checkoutDetails.BillingAddress.FirstName + " " + checkoutDetails.BillingAddress.LastName );
            collection.Add( "street_address", checkoutDetails.BillingAddress.Address1 );
            collection.Add( "street_address2", checkoutDetails.BillingAddress.Address2 );
            collection.Add( "city", checkoutDetails.BillingAddress.City );
            collection.Add( "country", checkoutDetails.BillingAddress.Country );
            collection.Add( "state", checkoutDetails.BillingAddress.State );
            collection.Add( "zip", checkoutDetails.BillingAddress.Zip );
            collection.Add( "email", checkoutDetails.Email );
            collection.Add( "phone", checkoutDetails.BillingAddress.Phone );

            if (DataAccessContext.Configurations.GetBoolValue( "2COTestMode" ))
            {
                collection.Add( "demo", "Y" );
            }

            return collection;
        }
    }
}
