﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Vevo.Domain.Base;

namespace Vevo.Domain.Products
{
    public class SpecificationGroup : ILocaleLoader, ICloneable
    {
        #region Private

        private string _specificationGroupID = "0";
        private string _name = String.Empty;

        private Culture _defaultCulture;
        private LocaleCollection<SpecificationGroupLocale> _locales;

        #endregion

        #region Public Properties

        public static SpecificationGroup Null
        {
            get { return NullSpecificationGroup.Instance; }
        }

        public virtual bool IsNull
        {
            get { return false; }
        }

        public string SpecificationGroupID { get { return _specificationGroupID; } set { _specificationGroupID = value; } }
        public string Name { get { return _name; } set { _name = value; } }
        public Culture DefaultCulture { get { return _defaultCulture; } set { _defaultCulture = value; } }
        public LocaleCollection<SpecificationGroupLocale> Locales { get { return _locales; } }

        //------------------------------------------------------------
        // Locale items
        public string Description
        {
            get { return Locales[DefaultCulture].Description; }
            set { Locales.GetNoLoad( DefaultCulture ).Description = value; }
        }
        public string DisplayName
        {
            get { return Locales[DefaultCulture].DisplayName; }
            set { Locales.GetNoLoad( DefaultCulture ).DisplayName = value; }
        }
        // End Locale items
        //------------------------------------------------------------

        #endregion



        #region Custom Properties
/*
        public IList<SpecificationItem> SpecificationItem
        {
            get
            {
                return DataAccessContext.OptionItemRepository.GetByOptionGroupID( DefaultCulture, OptionGroupID );
            }
        }
*/
        #endregion


        #region Public Constructors

        public SpecificationGroup( Culture defaultCulture )
        {
            _defaultCulture = defaultCulture;
            _locales = new LocaleCollection<SpecificationGroupLocale>( this );
        }

        #endregion

        #region ILocaleLoader Members

        public IList<ILocale> GetLocales()
        {
            List<ILocale> result = new List<ILocale>();

            foreach (SpecificationGroupLocale locale in DataAccessContext.SpecificationGroupRepository.GetLocales( SpecificationGroupID ))
                result.Add( locale );

            return result;
        }

        #endregion


        #region IClonable Members

        // Copy the whole aggregate (including locale)
        public object Clone()
        {
            SpecificationGroup newObject = (SpecificationGroup) MemberwiseClone();
            newObject._locales = (LocaleCollection<SpecificationGroupLocale>) Locales.Clone();

            return newObject;
        }

        #endregion


    }
}
