using System;
using System.Data;
using System.Collections.Generic;
using System.Configuration;

using Vevo.Domain.Configurations;
using Vevo.Domain.Orders;
using Vevo.Domain.Users;
using Vevo.Domain.Marketing;

namespace Vevo.Domain.Shipping.Custom
{
    /// <summary>
    /// Summary description for PerItemShippingMethod
    /// </summary>
    public class PerItemShippingMethod : CustomShippingMethod
    {
        #region Private

        private int GetItemCount( IList<ICartItem> cartItems )
        {
            int count = 0;
            foreach (ICartItem cartItem in cartItems)
            {
                if (cartItem.IsShippable())
                {
                    //if (cartItem.IsPromotion)
                    //{
                    //    int sumQty = 0;
                    //    foreach (PromotionSelectedItem item in cartItem.PromotionSelected.PromotionSelectedItems)
                    //    {
                    //        sumQty += item.GetPromotionProduct.Quantity;
                    //    }
                    //    count += cartItem.Quantity * sumQty;
                    //}
                    //else
                    //{
                        count += cartItem.Quantity;
                    //}
                }
            }
            return count;
        }

        private decimal GetPerItemShippingCost( CartItemGroup cartItemGroup )
        {
            int totalItemCounts = GetItemCount( GetNonOverriddenShippingCartItems( cartItemGroup ) );

            if (totalItemCounts > 0)
                return ShippingOption.ShippingCostFirstItem
                    + ((totalItemCounts - 1) * ShippingOption.ShippingCostNextItem);
            else
                return 0;
        }

        #endregion


        #region Constructors

        public PerItemShippingMethod(
            ShippingOption shippingOption,
            ShippingChoice selectedShippingChoice )     // ignored in non-real-time shipping option
            : base( shippingOption )
        {
        }

        #endregion


        #region Public Constances

        public const string ShippingOptionTypeName = "PerItem";

        #endregion


        #region Public Methods

        public override decimal GetShippingCost(
            CartItemGroup cartItemGroup,
            WholesaleStatus wholesaleStatus,
            decimal discount )
        {
            if (IsFreeShippingForNonOverriddenItems( cartItemGroup, wholesaleStatus ))
                return GetOverriddenShippingAmount( cartItemGroup );
            else
                return GetOverriddenShippingAmount( cartItemGroup ) + GetPerItemShippingCost( cartItemGroup );
        }

        #endregion
    }
}

