﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using Vevo.WebServices.Base;
using Vevo.WebServices.Security;
using Vevo.WebServices.SearchFilters;
using Vevo.WebAppLib;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain;
using System.Data;
using Vevo.Domain.Marketing;
using Vevo.Base.Domain;
using Vevo.Deluxe.Domain.Marketing;
using Vevo.Deluxe.Domain;

namespace Vevo.WebServices.CommissionPayments
{
    public class CommissionPaymentService : ICommissionPaymentService
    {
        private CommissionPaymentConstraints ConvertCommissionPaymentToConstraints( AffiliatePayment commissionPayment )
        {
            CommissionPaymentConstraints constraints = new CommissionPaymentConstraints();
            constraints.AffiliateCode = commissionPayment.AffiliateCode;
            constraints.AffiliatePaymentID = commissionPayment.AffiliatePaymentID;
            constraints.Amount = commissionPayment.Amount;
            constraints.PaidDate = commissionPayment.PaidDate;
            constraints.PaymentNote = commissionPayment.PaymentNote;

            return constraints;
        }

        public CommissionPaymentListResponseMessage ListCommissionPayments(CommissionPaymentListRequestMessage ListMessage )
        {
            CommissionPaymentListResponseMessage responseMessage = new CommissionPaymentListResponseMessage();
            if ( WcfSessionPool.GetSession( ListMessage.Ticket ) == null )
            {
                responseMessage.ErrorMessage = "Error: Authentication is failed.";
                return responseMessage;
            }
            try
            {
                SearchFilter searchFilter = SearchFilterUtilities.ConvertConstraintToSearchFilter( ListMessage.SearchFilter );
                int commissionPaymentCount = 0;
                IList<AffiliatePayment> commissionPayment_result = DataAccessContextDeluxe.AffiliatePaymentRepository.SearchAffiliatePayment(
                    ListMessage.AffiliateCode, "AffiliatePaymentID", searchFilter,
                    ListMessage.SearchFilter.StartIndex, ListMessage.SearchFilter.EndIndex, out commissionPaymentCount );
                responseMessage.CommissionPayments = new List<CommissionPaymentConstraints>();
                foreach ( AffiliatePayment commissionPayment in commissionPayment_result )
                {
                    responseMessage.CommissionPayments.Add( ConvertCommissionPaymentToConstraints( commissionPayment ) );
                }
            }
            catch ( Exception e )
            {
                WebUtilities.LogError( e );
                responseMessage.ErrorMessage = "Error: Cannot get commission payments from repository";
            }

            return responseMessage;
        }
    }
}
