﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using Vevo.WebServices.Base;
using Vevo.WebServices.Security;
using Vevo.WebServices.SearchFilters;
using Vevo.WebAppLib;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain;
using System.Data;
using Vevo.Base.Domain;

namespace Vevo.WebServices.Currencies
{
    public class CurrencyService : ICurrencyService
    {
        private CurrencyConstraints ConvertCurrencyToConstraints( Currency currency )
        {
            CurrencyConstraints constraints = new CurrencyConstraints();
            constraints.ConversionRate = currency.ConversionRate;
            constraints.CurrencyCode = currency.CurrencyCode;
            constraints.CurrencySymbol = currency.CurrencySymbol;
            constraints.Enabled = currency.Enabled;
            constraints.Name = currency.Name;

            return constraints;
        }

        public CurrencyListResponseMessage ListCurrencies( CurrencyListRequestMessage ListMessage )
        {
            CurrencyListResponseMessage responseMessage = new CurrencyListResponseMessage();
            if ( WcfSessionPool.GetSession( ListMessage.Ticket ) == null )
            {
                responseMessage.ErrorMessage = "Error: Authentication is failed.";
                return responseMessage;
            }
            try
            {
                SearchFilter searchFilter = SearchFilterUtilities.ConvertConstraintToSearchFilter(
                                ListMessage.SearchFilter );
                int currencyCount = 0;
                IList<Currency> currency_result = DataAccessContext.CurrencyRepository.SearchCurrency( "CurrencyCode",
                    searchFilter, ListMessage.SearchFilter.StartIndex, ListMessage.SearchFilter.EndIndex, out currencyCount );
                responseMessage.Currencies = new List<CurrencyConstraints>();
                foreach ( Currency currency in currency_result )
                {
                    responseMessage.Currencies.Add( ConvertCurrencyToConstraints( currency ) );
                }
            }
            catch ( Exception e )
            {
                WebUtilities.LogError( e );
                responseMessage.ErrorMessage = "Error: Cannot get currencies from repository";
            }

            return responseMessage;
        }
    }
}
