﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using Vevo.Domain;
using Vevo.Domain.Security;
using Vevo.Shared.Utilities;
using Vevo.Shared.WebUI;
using Vevo.WebAppLib;
using Vevo.Domain.Stores;

namespace Vevo.WebUI.BaseControls
{
    public class BaseBlogMasterPage : Vevo.WebUI.International.BaseLanguageMasterPage
    {
        #region Private

        private bool IsPageErrorException()
        {
            foreach (string page in _PageErrorException)
            {
                if (String.Compare(GetPageName(Request.Url.AbsolutePath), page, true) == 0)
                {
                    return true;
                }
            }

            return false;
        }

        private string[] _PageErrorException = new string[]
        {
            "GenericError.aspx"
         };

        private string GetPageName(string url)
        {
            string[] splitURL = url.Split('/');
            int upperBound = splitURL.GetUpperBound(0);

            return splitURL[upperBound].ToString();
        }

        private bool IsTransaction
        {
            get
            {
                if (!String.IsNullOrEmpty(Request.QueryString["IsTransaction"]))
                    return ConvertUtilities.ToBoolean(Request.QueryString["IsTransaction"]);
                else
                    return false;
            }
        }

        private string OrderID
        {
            get
            {
                if (!String.IsNullOrEmpty(Request.QueryString["OrderID"]))
                    return Request.QueryString["OrderID"];
                else
                    return "0";
            }
        }

        private void SSLVerifyAndRedirect()
        {
            string newUrl;
            SslPolicy sslPolicy = new SslPolicy();

            // Need to get user-friendly url (e.g. CatalogA-list.aspx 
            // instead of Catalog.aspx?CatalogName=CatalogA.)
            if (sslPolicy.RequiresRedirect(UrlPath.GetDisplayedUrl(), out newUrl))
            {
                Response.Redirect(newUrl);
            }
        }

        private void AddMetaItem(string name, string content)
        {
            WebUtilities.PageAddMeta(Page, name, content);
        }

        private void SetMetaAndTitle()
        {
            string currentPage = Request.Url.AbsolutePath.ToLower();

            if (!currentPage.Contains("bloglist.aspx") &&
                !currentPage.Contains("blogdetails.aspx"))
            {
                AddMetaItem("description", DataAccessContext.Configurations.GetValue(StoreContext.Culture.CultureID, "BlogPageTitle"));
                AddMetaItem("keywords", DataAccessContext.Configurations.GetValue(StoreContext.Culture.CultureID, "BlogPageTitle"));
            }
        }

        private void SetCanonicalization()
        {
            //string currentPage = Request.Url.AbsolutePath.ToLower();

            string pageAndQueryUrl = Request.Url.ToString().Substring(UrlPath.StorefrontUrl.Length - 1);

            if (pageAndQueryUrl.ToLower().Contains("-details.aspx") ||
                pageAndQueryUrl.ToLower().Contains("-list.aspx") ||
                pageAndQueryUrl.ToLower().Contains("-items.aspx") ||
                pageAndQueryUrl.ToLower().Contains("catalog.aspx") ||
                pageAndQueryUrl.ToLower().Contains("department.aspx") ||
                pageAndQueryUrl.ToLower().Contains("product.aspx"))
            {
                WebUtilities.PageCannonicalization(Page, pageAndQueryUrl, StoreContext.Culture);
            }
        }

        private void AddIconLink()
        {
            HtmlLink link = new HtmlLink();
            link.Attributes.Add("rel", "shortcut icon");
            link.Attributes.Add("type", "image/vnd.microsoft.icon");
            link.Href = "~/favicon.ico";
            Page.Header.Controls.Add(link);
        }

        private void RegisterGoogleAnalytics()
        {
            string script;
            if (IsTransaction)
                script = GoogleAnalyticUtilities.RegisterTransactionScript(OrderID);
            else
                script = GoogleAnalyticUtilities.RegisterTrackingScript();
            ScriptManager.RegisterStartupScript(this, this.GetType(), "GoogleAnalytics", script, false);
        }

        private void ConfigAjaxAsyncPostBackError()
        {
            if (DataAccessContext.Configurations.GetBoolValueNoThrow("ShowDetailAjaxErrorMessage"))
            {
                ScriptManager.GetCurrent(Page).AsyncPostBackError += ScriptManager_AsyncPostBackError;
            }
        }

        private void RegisterClientScriptLibraries()
        {
            ScriptManager.RegisterClientScriptInclude(Page, typeof(Page), "ForCallOut", "ClientScripts/Utilities.js");                 
            ScriptManager.RegisterClientScriptInclude(Page, typeof(Page), "jquery", "ClientScripts/JQuery/jquery-1.8.1.min.js");
            ScriptManager.RegisterClientScriptInclude(Page, typeof(Page), "jquerycycle", "ClientScripts/JQuery/jquery.cycle.all.min.js");
            ScriptManager.RegisterClientScriptInclude(Page, typeof(Page), "jquerypop", "ClientScripts/JQuery/jquery.pop.js");
        }

        private void BaseBlogMasterPage_Load(object sender, EventArgs e)
        {
            if (!DataAccessContext.ApplicationSettings.InstallCompleted && !IsPageErrorException())
            {
                Response.Redirect("~/Install/PermissionCheck.aspx");
            }

            DatabaseConnectionService connectionService = new DatabaseConnectionService();
            if (!connectionService.TryConnect())
            {
                throw new Exception("Database Connection Error.There is an error while attempting to connect to the database. Please verify your connection string in the file.");
            }

            //DataAccessContext.EnableConfigurations( true );
            //ConfigurationHelper.ApplyConfigurations();

            SSLVerifyAndRedirect();

            SetMetaAndTitle();

            AddIconLink();

            RegisterGoogleAnalytics();

            ConfigAjaxAsyncPostBackError();

            RegisterClientScriptLibraries();

            

            if (KeyUtilities.IsMultistoreLicense())
            {
                SetCanonicalization();
            }
        }

        #endregion


        #region Constructors

        public BaseBlogMasterPage()
        {
            Load += new EventHandler(BaseBlogMasterPage_Load);
        }

        #endregion


        #region Protected

        protected void ScriptManager_AsyncPostBackError(object sender, AsyncPostBackErrorEventArgs e)
        {
            ScriptManager.GetCurrent(Page).AsyncPostBackErrorMessage =
                String.Format("Exception Type: {0}\n\nMessage: {1}\n\nMethod: {2}\n\nStack Trace:\n{3}",
                    e.Exception.GetType().ToString(),
                    e.Exception.Message,
                    e.Exception.TargetSite,
                    e.Exception.StackTrace);
        }

        protected bool IsRestrictedAccessPage()
        {
            string path = Request.Url.AbsolutePath.ToLower();
            bool isAuthPage = true;

            if (path.Contains( "userlogin.aspx" ) || path.Contains( "register.aspx" ) || path.Contains( "forgotpassword.aspx" ) ||
                path.Contains( "resetpassword.aspx" ) || path.Contains( "passwordrecoveryfinished.aspx" ) ||
                path.Contains( "registerfinish.aspx" ) || path.Contains( "affiliatelogin.aspx" ))
            {
                isAuthPage = false;
            }

            return isAuthPage;
        }

        protected string AffiliateCode
        {
            get
            {
                if (!String.IsNullOrEmpty( Request.QueryString["AffiliateCode"] ))
                    return Request.QueryString["AffiliateCode"];
                else
                    return String.Empty;
            }
        }

        #endregion
    }
}
