using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using Vevo.Data;
using Vevo.Domain;
using Vevo.Domain.Marketing;
using Vevo.Domain.Orders;
using Vevo.Domain.Payments;
using Vevo.Domain.Products;
using Vevo.Domain.Stores;
using Vevo.Domain.SystemServices;
using Vevo.Domain.Users;
using Vevo.Shared.Domain.SystemServices;
using Vevo.Shared.WebUI;
using Vevo.WebAppLib;
using Vevo.Deluxe.Domain;
using Vevo.Deluxe.Domain.GiftRegistry;
using Vevo.Deluxe.Domain.CustomerReward;
using Vevo.Shared.Utilities;
using Vevo.Deluxe.Domain.Users;
using Vevo.Deluxe.Domain.Products;

namespace Vevo.WebUI.Orders
{
    /// <summary>
    /// Summary description for Order
    /// </summary>
    public class OrderNotifyService
    {
        #region Private
        private string _orderID;


        private void SendEmailToCustomer(
            string customerEmail,
            string orderID,
            string subject,
            string body )
        {
            IMailSendService mailSendService = new StoreMailSendService();
            OrderRepository orderRepository = new OrderRepository();
            Order order = orderRepository.GetOne( orderID );
            Store store = DataAccessContext.StoreRepository.GetOne( order.StoreID );
            string companyEmail = DataAccessContext.Configurations.GetValue( "CompanyEmail", store );
            WebUtilities.SendHtmlMail(
                companyEmail,
                customerEmail,
                subject,
                body,
                store );
        }

        private void SendEmailToMerchant(
            string customerUserName,
            string customerEmail,
            string orderID,
            string subject,
            string body )
        {
            StringBuilder sb = new StringBuilder();

            sb.Append( "Merchant Order Confirmation" );
            sb.Append( "<br/><br/>" );
            sb.Append( "Customer User Name: " );
            sb.Append( customerUserName );
            sb.Append( "<br/>" );
            sb.Append( "Customer Email Address: " );
            sb.Append( customerEmail );
            sb.Append( "<br/>" );
            sb.Append( "******************************************" );
            sb.Append( "<br/><br/><br/>" );
            sb.Append( body );

            IMailSendService mailSendService = new StoreMailSendService();
            OrderRepository orderRepository = new OrderRepository();
            Order order = orderRepository.GetOne( orderID );
            Store store = DataAccessContext.StoreRepository.GetOne( order.StoreID );
            string companyEmail = DataAccessContext.Configurations.GetValue( "CompanyEmail", store );
            WebUtilities.SendHtmlMail(
                companyEmail,
                companyEmail,
                subject,
                sb.ToString(),
                store );
        }

        private void SendMailToGiftRegistryOwner( string giftRegistryID )
        {
            GiftRegistry giftRegistry = DataAccessContextDeluxe.GiftRegistryRepository.GetOne( giftRegistryID );
            Order order = DataAccessContext.OrderRepository.GetOne( _orderID );

            if (!giftRegistry.NotifyNewOrder)
                return;

            string subjectMail;
            string bodyMail;

            EmailTemplateTextVariable.ReplaceGiftRegistryNewOrderText( order, out subjectMail, out bodyMail );

            string ownerEmail = DataAccessContext.CustomerRepository.GetOne( giftRegistry.CustomerID ).Email;

            IMailSendService mailSendService = new StoreMailSendService();

            mailSendService.SendHtmlMail(
                DataAccessContext.Configurations.GetValue( "CompanyEmail" ),
                ownerEmail,
                subjectMail,
                bodyMail );
        }

        private string GetLinkProductDownload( string productID, string userName, string orderItemID )
        {
            DownloadKey downloadKey = new DownloadKey( productID, userName, orderItemID );
            OrderItem orderItem = DataAccessContext.OrderItemRepository.GetOne( orderItemID );
            string key = downloadKey.GetProductDownloadKey();
            string storefrontUrl = new StoreRetriever().GetStorefrontUrlByOrderID( orderItem.OrderID ) + "/";

            if (!KeyUtilities.IsMultistoreLicense())
                storefrontUrl = UrlPath.StorefrontUrl;

            string link = String.Format( "{0}FileDownloadManager.aspx?", storefrontUrl )
                + "ProductID=" + HttpContext.Current.Server.UrlEncode( productID )
                + "&OrderItemID=" + HttpContext.Current.Server.UrlEncode( orderItemID )
                + "&Key=" + HttpContext.Current.Server.UrlEncode( key );

            return link;
        }

        #endregion


        #region Constructors

        public OrderNotifyService( string orderID )
        {
            _orderID = orderID;
        }

        #endregion


        #region Public Methods

        public string OrderID
        {
            get
            {
                return _orderID;
            }
        }

        public string PaymentMethod
        {
            get
            {
                Order order = DataAccessContext.OrderRepository.GetOne( _orderID );
                return order.PaymentMethod;
            }
        }

        public void SendOrderEmail()
        {
            Order order = DataAccessContext.OrderRepository.GetOne( _orderID );

            string subjectMail;
            string bodyMail;

            EmailTemplateTextVariable.ReplaceOrderConfirmationText( order, out subjectMail, out bodyMail );

            SendEmailToCustomer( order.Email, _orderID, subjectMail, bodyMail );
            SendEmailToMerchant(
                order.UserName,
                order.Email,
                _orderID,
                subjectMail,
                bodyMail );

            SendMailToGiftRegistryOwner( order.GiftRegistryID );
        }

        public Exception SendOrderEmailNoThrow()
        {
            try
            {
                SendOrderEmail();
            }
            catch (Exception ex)
            {
                return ex;
            }

            return null;
        }

        public void SendGiftCodeMail( Order order )
        {
            string email = order.Email;
            IList<GiftCertificate> giftList = DataAccessContext.GiftCertificateRepository.GetAllByOrderID( OrderID );

            if (giftList.Count > 0)
            {
                Store store = DataAccessContext.StoreRepository.GetOne( order.StoreID );
                string siteName = DataAccessContext.Configurations.GetValue(
                    CultureUtilities.StoreCultureID, "SiteName", store );

                string GiftUrl
                    = new StoreRetriever().GetStorefrontUrlByOrderID( OrderID ) + "/GiftCertificate.aspx?Code=";

                if (!KeyUtilities.IsMultistoreLicense())
                    GiftUrl = UrlPath.StorefrontUrl + "GiftCertificate.aspx?Code=";

                StringBuilder giftBody = new StringBuilder();
                for (int i = 0; i < giftList.Count; i++)
                {
                    giftBody.Append( "<br/>" );
                    giftBody.Append( "Gift Certitficate value : "
                        + StoreContext.Currency.FormatPrice( giftList[i].RemainValue ) );
                    giftBody.Append( "<br/>" );
                    giftBody.Append( GiftUrl + giftList[i].UrlParameter );
                    giftBody.Append( "<br/><br/>" );
                }

                string subjectMail;
                string bodyMail;

                EmailTemplateTextVariable.ReplaceGiftCertificateCodeText( siteName, order, giftBody, out subjectMail, out bodyMail );

                SendEmailToCustomer( email, _orderID, subjectMail, bodyMail );
            }
        }

        public void ProcessPaymentComplete()
        {
            if (!StoreContext.ShoppingCart.ContainsRecurringProduct())
            {
                Order order = DataAccessContext.OrderRepository.GetOne( _orderID );
                SendGiftCodeMail( order );
                order.PaymentComplete = true;
                CustomerRewardPoint.UpdateRewardPoint( order );
                order.IsPointApplied = true;
                UpdateCustomerSubscription( order );
                order.IsSubscriptionApplied = true;
                DataAccessContext.OrderRepository.Save( order );
                SendDownloadEmailByOrderID();

                SendSubscriptionContentEmailByOrderID();
            }
        }

        public void UpdateCustomerSubscription( Order order )
        {
            Customer customer;
            GiftRegistry gift = DataAccessContextDeluxe.GiftRegistryRepository.GetOne( order.GiftRegistryID );
            if (gift.IsNull)
                customer = DataAccessContext.CustomerRepository.GetOne( order.CustomerID );
            else
                customer = DataAccessContext.CustomerRepository.GetOne( gift.CustomerID );

            if (customer.IsNull)
                return;

            for (int rowIndex = 0; rowIndex < order.OrderItems.Count; rowIndex++)
            {
                OrderItem item = order.OrderItems[rowIndex];
                Product product = DataAccessContext.ProductRepository.GetOne(
                    StoreContext.Culture, item.ProductID, order.StoreID );

                ProductSubscription productSubscription = new ProductSubscription( product.ProductID );

                if (productSubscription.IsSubscriptionProduct())
                {
                    CustomerSubscription subscription = new CustomerSubscription();
                    subscription = subscription.GetCustomerSubscriptionByLevelID( productSubscription.ProductSubscriptions[0].SubscriptionLevelID, order.CustomerID );

                    bool isNew = false;

                    if (subscription == null)
                        isNew = true;
                    else
                        isNew = false;


                    CustomerSubscription customerSubscription = new CustomerSubscription();
                    customerSubscription.CustomerID = customer.CustomerID;
                    customerSubscription.SubscriptionLevelID = productSubscription.ProductSubscriptions[0].SubscriptionLevelID;
                    customerSubscription.StartDate = DateTime.Today;
                    customerSubscription.EndDate = DateTime.Today.AddDays( productSubscription.ProductSubscriptions[0].Range );
                    customerSubscription.IsActive = true;
                    
                    customerSubscription.SetCustomerSubscription( customerSubscription, isNew );
                    DataAccessContextDeluxe.CustomerSubscriptionRepository.Save( customer.CustomerID, customerSubscription.CustomerSubscriptions );
                }
            }
        }

        public void ProcessPaymentFailed()
        {
            Order order = DataAccessContext.OrderRepository.GetOne( _orderID );
            order.PaymentComplete = false;
            DataAccessContext.OrderRepository.Save( order );
        }

        public void SendSubscriptionContentEmailByOrderID()
        {
            Order order = DataAccessContext.OrderRepository.GetOne( _orderID );
            string userName = order.UserName;

            IList<OrderItem> orderItemList = DataAccessContext.OrderItemRepository.GetByOrderID( _orderID );
            string result = String.Empty;
            Store store = DataAccessContext.StoreRepository.GetOne( order.StoreID );
            string companyEmail = DataAccessContext.Configurations.GetValue( "CompanyEmail", store );

            int subscriptionLevel = 0;

            for (int rowIndex = 0; rowIndex < orderItemList.Count; rowIndex++)
            {
                OrderItem item = orderItemList[rowIndex];
                Product product = DataAccessContext.ProductRepository.GetOne(
                    StoreContext.Culture, item.ProductID, order.StoreID );

                ProductSubscription productSubscription = new ProductSubscription( product.ProductID );

                if (productSubscription.IsSubscriptionProduct())
                {
                    int level = DataAccessContextDeluxe.SubscriptionLevelRepository.GetOne( productSubscription.ProductSubscriptions[0].SubscriptionLevelID ).Level;
                    if (level > subscriptionLevel)
                        subscriptionLevel = level;
                }
            }

            if (subscriptionLevel != 0)
            {
                bool success = false;
                string subjectMail;
                string bodyMail;
                string sendEmail;

                GiftRegistry gift = DataAccessContextDeluxe.GiftRegistryRepository.GetOne( order.GiftRegistryID );
                if (gift.IsNull)
                    sendEmail = order.Email;
                else
                {
                    Customer giftCustomer = DataAccessContext.CustomerRepository.GetOne( gift.CustomerID );
                    sendEmail = giftCustomer.Email;
                }

                if (EmailTemplateTextVariable.ReplaceSubscriptionContentLinkText(
                    order.OrderID, subscriptionLevel.ToString(), order.StoreID, out subjectMail, out bodyMail ))
                {
                    WebUtilities.SendHtmlMail(
                        companyEmail,
                        sendEmail,
                        subjectMail,
                        bodyMail,
                        store );

                    success = true;
                }

                if (!success)
                {
                    WebUtilities.SendMail(
                        NamedConfig.AutoSenderEmail,
                        companyEmail,
                        "Error sending subscription content mail",
                        "Cannot send subscription content mail for order ID " + _orderID + ".\n" +
                        "Please check your email templates.",
                        store );
                }
            }
        }

        public void SendDownloadEmailByOrderID()
        {
            Order order = DataAccessContext.OrderRepository.GetOne( _orderID );
            string userName = order.UserName;

            IList<OrderItem> orderItemList = DataAccessContext.OrderItemRepository.GetByOrderID( _orderID );
            string result = String.Empty;
            Store store = DataAccessContext.StoreRepository.GetOne( order.StoreID );
            string companyEmail = DataAccessContext.Configurations.GetValue( "CompanyEmail", store );

            for (int rowIndex = 0; rowIndex < orderItemList.Count; rowIndex++)
            {
                OrderItem item = orderItemList[rowIndex];
                Product product = DataAccessContext.ProductRepository.GetOne(
                    StoreContext.Culture, item.ProductID, order.StoreID );

                if (product.IsDownloadable)
                {
                    result += item.Name + " Link:    " +
                         GetLinkProductDownload( product.ProductID, userName, item.OrderItemID ) + "\n";
                }
            }

            if (result != String.Empty)
            {
                bool success = false;
                string subjectMail;
                string bodyMail;
                string sendEmail;

                GiftRegistry gift = DataAccessContextDeluxe.GiftRegistryRepository.GetOne( order.GiftRegistryID );
                if (gift.IsNull)
                    sendEmail = order.Email;
                else
                {
                    Customer giftCustomer = DataAccessContext.CustomerRepository.GetOne( gift.CustomerID );
                    sendEmail = giftCustomer.Email;
                }

                if (EmailTemplateTextVariable.ReplaceDownloadFileLinkText( result, order.StoreID, out subjectMail, out bodyMail ))
                {
                    WebUtilities.SendHtmlMail(
                        companyEmail,
                        sendEmail,
                        subjectMail,
                        bodyMail,
                        store );

                    success = true;
                }

                if (!success)
                {
                    WebUtilities.SendMail(
                        NamedConfig.AutoSenderEmail,
                        companyEmail,
                        "Error sending download link",
                        "Cannot send download link for order ID " + _orderID + ".\n" +
                        "Please check your email templates.",
                        store );
                }
            }

        }

        #endregion
    }
}
